%------------------------------------------------------------------------
% Routine         : aafile.m
%
% Description     : M-file for determining focal regions of an annular array 
%
%
% Language        : Matlab 4.0
% Originator, date	: Vingmed Sound A/S, SH            
% Modified by		: SH
% Version no, date	: 3.0, SH 18. May 1993           
%                	: 3.1, SH 20. July 1993 - changed resolution plot
%                         3.2, SH 2. Sept 1993 - lists c (m/s) since it is
%                                        often confused with dome parameter 
%			  3.3, SH 7. Oct 1993 - lists recommended range for fixed focus 
%			  3.4, MM 10. Jan 1994 - changed on-axis pressure 
%					from 6.131c, to 6.123, Angelsen.
%                         3.5, SH 18. Jan 1994 - added c_foc for constant value for c  
%                         3.6  FT 17. Feb 1994 - c_foc = excitation(14)
%                         3.7  SH 02.10.96     - alfa is now in 1/m*MHz as input
%                              SH 18.09.06 - default value for rangemax,
%                                           added return, correced for numerical problems
%                                           near focal point
%                                                           
%                                            
% Called by       : usimcnf.m
% Routines called : 
% Load files      : 
%
%------------------------------------------------------------------------
%

function aafile(transducer, excitation, media, flagg)
	
	disp(' ')
        disp('*** Equal-area annular array focusing in homogenous medium ***')
        disp(' ')

	if transducer(18) ~= 0 | (flagg(2) ~= 2 & flagg(2) ~= 3)
		disp('Can only analyze equal-area annular arrays!')
		return
	end
%
% Constant
%
	c_foc = excitation(14);
%
% Variables initialized by system:
%
	f = excitation(1);
	d = transducer(1); % diameter of xdcr
	r = transducer(9); % focal pt of xdcr
	Nel = transducer(2);
%	c = media(1,2);
	c = c_foc;
	alfa = media(1,9);
	lambda = c/f;
        disp('Probe parameters:')
        disp(['      Frequency: ',num2str(f*1e-6),' MHz'])
        disp(['       Aperture: ',num2str(d*1e3),' mm'])
        disp(['    Fixed focus: ',num2str(r*1e3),' mm'])
        disp(['     # elements: ',num2str(Nel)])
        disp(' ')
        disp('Medium parameters:')
        disp([' Sound velocity: ',num2str(c),' m/s'])
	alfadB = alfa/(10*log(10));
	disp(['Derating factor: ',num2str(alfadB),' dB/cm/MHz = '...
	     ,num2str(alfa),' 1/m*MHz'])  
%
% Parameters of CFM front-end:
%
        Nfoc = 4;
	taulim = 300e-9;

	disp(' ')
	disp('Front-end parameters:')
%	disp(['# Focal zones: ',num2str(Nfoc)])
	disp(['    Max delay: ',num2str(taulim*1e6),' microsec'])
%
% Find boundaries between elements:
%
        nel = 1:Nel;
        xel = (d/2)*sqrt(nel/Nel);
%
% find tau(1,infinite) - delay over one element at infinity:
%
        tau1inf = d*d/(8*Nel*r*c);
%
% find tau - delay over one element as a function of range
%
	rangemin = 0.2e-2;
	rangedelta = 0.1e-2;
	rangemax = 1e-2*input(' Max range for plotting [cm]: ');
    if isempty(rangemax),
        rangemax = 30e-2;
    end
    range = rangemin:rangedelta:rangemax;
    Nrange = max(size(range));
    tau1 = tau1inf*(range-r)./range;
	tau1 = clipl(tau1, -1000e-9);

%
% plot lines to indicate 
% approximate boundaries for useful range:
%
	tau70 = (70/360)*(1e9/f)*ones(size(range));
        tau94 = (94/360)*(1e9/f)*ones(size(range));
        tau132 = (132/360)*(1e9/f)*ones(size(range));       
        tau159 = (159/360)*(1e9/f)*ones(size(range));       
%
% Relationship is:    dB loss,  max phase error (degrees)
%                       0.5             70
%                       1.0             94
%                       2.0             132
%                       3.0             159 
%
	subplot(2,2,1)
        plot(range*1000, tau1*1e9, ... 
        range*1000, tau94,'-.', range*1000, -tau94,'-.', ...
        range*1000, tau132,'-.', range*1000, -tau132,'-.', ...         
        range*1000, tau159,'-.', range*1000, -tau159,'-.')
     	xlabel(['Range [mm], fixed focus ',num2str(r*1e3), ' mm'])
        title('Element delay error [ns], -1,-2,-3 dB loss')
        ylabel([num2str(f*1e-6),' MHz, ',num2str(Nel), ' rings, aperture ',num2str(d*1e3), ' mm'])
%        text((0/Nrange)*1000*rangemax, -tau94(1),'-1 dB')
%        text((100/Nrange)*1000*rangemax, tau94(1), '-1 dB')
%        text((15/Nrange)*1000*rangemax, -tau132(1),'-2 dB')
%        text((115/Nrange)*1000*rangemax, tau132(1), '-2 dB')
%        text((30/Nrange)*1000*rangemax, -tau159(1),'-3 dB')
%        text((130/Nrange)*1000*rangemax, tau159(1), '-3 dB')
        datetime;
%       text(0.68, 0.15, datetime,'sc')
        grid
	v = axis;
	axis([v(1) v(2) -2*tau159(1) 2*tau159(1)]);
%
% find max delay:
%
	tautotal = tau1*(Nel-1);
	taumax = max(tautotal);
	taumax = clipl(taumax, -2000e-9);
	subplot(2,2,2)
	plot(range*1000, tautotal*1e9, ...
    	range*1000, taulim*1e9*ones(size(range)),'-.', range*1000, -taulim*1e9*ones(size(range)),'-.')
	temp = [num2str(taulim*1e9),' ns'];
	text(0,1e9*taulim, temp);
	text(0,-1e9*taulim,['-', temp]);
     	xlabel(['Range [mm], ',datetime])
        title('Max beamformer delay')
	ylabel('[ns]')
	grid
	axis([v(1) v(2) -2e9*taulim(1) 2e9*taulim(1)]);
%
% check if max delay in frontend is exceeded:
%
	if (taumax > taulim),
		disp(['Max delay in front-end is exceeded: ',num2str(taumax*1e9),' ns']) 
	end

%
% plot expected -12 dB (kx = 2) or null-to-null (kx= 2.44) resolution as a function of depth
%
	kx = 2.0;
	if Nel == 1,
		resolution = ( kx*ones(size(range))'*lambda/(2*xel) ) .* range';
	else
		resolution = (kx*ones(size(range))'*(lambda*ones(size(xel))./(2*xel))) .* ... 	
			(range'*ones(size(xel)));
	end
%
% find useful range of probe based on phase error over a single element
%
	n5a = max(find(tau1 < -tau70(1)*1e-9));
	n5b = max(find(tau1 <= tau70(1)*1e-9));
	n1a = max(find(tau1 < -tau94(1)*1e-9));
	n1b = max(find(tau1 <= tau94(1)*1e-9));
	n2a = max(find(tau1 < -tau132(1)*1e-9));
	n2b = max(find(tau1 <= tau132(1)*1e-9));
	n3a = max(find(tau1 < -tau159(1)*1e-9));
	n3b = max(find(tau1 <= tau159(1)*1e-9));

%
% resolution plot: useful range is in solid line, rest in dotted
%
if n5a==n5b
  disp('Not a useful range...')
  disp('aafile aborted!')
  return
end
 	subplot(2,2,3)
	plot(range*1000, 1000*resolution,':', ...
		range(n1a:n1b)*1000,1000*resolution(n1a:n1b,:),'--', ...
		range(n5a:n5b)*1000,1000*resolution(n5a:n5b,:),'-' )
%		range(n1a:n1b)*1000,1000*resolution(n1a:n1b,:),'-.' , ...
%		range(n2a:n2b)*1000,1000*resolution(n2a:n2b,:),'-.', ...
	 	
	xlabel('Range [mm]')
	title('One-way lateral resolution (-12 dB)')
	ylabel('[mm]')
%   title([num2str(f*1e-6),' MHz, ',num2str(Nel), ' rings, aperture ',num2str(d*1e3), ' mm'])
	[mm,nn] = size(resolution);
	for nnn = 1:Nel
		if nnn == 1,
			text(rangemax*1000*0.85, resolution(mm,nnn)*1000,[num2str(nnn),' ring'])
		else
			text(rangemax*1000*0.85, resolution(mm,nnn)*1000,[num2str(nnn),' rings'])
	
		end
	end
	grid
        datetime;
%        text(0.68, 0.15, datetime,'sc')
%
% On-axis pressure, eq. 6.131c, Angelsen
% 12. Nov 1992 SH
% better to use 6.123 3.12.93
%
%	Fresnel parameter
	S = 4*r*c/(d*d*f);
	disp(['Fresnel parameter: ',num2str(S)])
	rnearfar = d*d*f/(4*c);
	disp(['Recommended range for fixed focus: ',num2str(500*rnearfar),' - ', ...
		num2str(1000*rnearfar),' mm, (S=0.5 - 1.0)'])
%
% 10. Jan 1994: pressure (6.123), Angelsen instead of (6.131)
%
%(6.131)arg = pi*(-1+ones(size(range))*r./range) / (2*S);
%(6.131)pressure = (ones(size(range))*r./range) .* sin(eps + arg)./(eps + arg);
%
	
	e	= r - sqrt(r^2 - (d/2)^2);
	Ra	= sqrt(range.^2 + 2*e*(ones(size(range))*r -  range));
	arg	= (pi/lambda)*(Ra - range);
	pressure= (r/e) * ...
		((Ra - range + eps)./(ones(size(range))*r - range + eps))...
		.*  (sin(eps + arg)./(eps + arg));
    %
    % possible numerical problem at range near r (focal pt of xdcr)
    %
	%%if any(range==r) % removed 18.09.2006
		[tmp,ind]	= min(abs(range - r));
		pressure(ind)	= 1;
	%%end 
%
% derating factor (one way) is alfa in 1/m*MHz
% beta = frequency dependency is neglected
%
% 7. Jan 1993: 0.5*alfa since if used for a pressure curve
%
	alfaprime = alfa*1e-6;
%
% approx value for max pressure
%
	rmax = r/(1+S/2);
	pmax = max(pressure);
	subplot(2,2,4)

%
% intensity, eq 6.123, 6.131 Angelsen
%
 	rho = 1000; 
	Z0 = rho * c;
	intensity =  (1/(2*Z0))*(pi/S)^2*(pressure.^2);
	imax = 10*log10(max(intensity)); 
	ii = 10*(fix(imax/10) + 1);
	
%
% 7. Jan 1993: alfaprime since this is an intensity curve
%
	derated = intensity.*exp(-alfaprime*f*range);
	plot(1000*range,10*log10(intensity),'-', ...
	1000*range,10*log10(derated),'-.', ...
             1000*rmax*ones(size(range)), ...
	     [imax-3 imax*ones(min(size(pressure)),max(size(pressure))-2) imax+3])

	
	title(['On-axis intensity, focus = ',num2str(r*1000),' mm'])
	ylabel(['[dB] --- water, -.- ',num2str(alfadB),' dB/cm*MHz']) 	
	xlabel(['Range [mm], Fresnel ',num2str(S),', c ',num2str(c),' m/s'])

%	title(['On-axis intensity, ',num2str(f*1e-6),' MHz, aperture ',num2str(d*1e3), ' mm'])
%	ylabel(['dB, - water, -. ',num2str(alfadB),' dB/cm*MHz']) 	
%	xlabel(['Range [mm], fixed focus ',num2str(r*1e3), ' mm, Fresnel par. ',num2str(S)])
%	datetext;
 	grid
	axis([0 1000*rangemax -60+ii ii]);
%
% find derated intensity maximum and position
%
    [de_max, de_index] = max(derated);
	disp(['Derated intensity maximum at ',num2str(1000*range(de_index)), ... 
		' mm, value ', num2str(de_max),', ',num2str(10*log10(de_max)),' dB']);
return
