%----------------------------------------------------------------
% Routine     : curved.m 
%
% Description : Analyzes curved and linear (unsteered) arrays
% 		and finds relationship between ROC, f-number and
% 		maximum useable angle of each element/or element pitch
%               
% Language    : Matlab V4.2
% Written by  : Dept. of Informatics, Univ. Oslo, Sverre Holm
%
% Version no. : 1.0 27.03.96 SH  standalone version
%		2.0 17.07.96 SH  Ultrasim version
%               2.1 30.07.96 SH  f-number is now range/chord for curved array
%               2.2 20.08.96 SH  Corrected equation for b after input from
%                                S.Bergstoel, added date on plot.
%               2.3 14.05.97 SH  Improved model for element angularity
%                                had to add global variables
%               2.4 22.05.97 SH  Added plot of f-stop vs depth
%                                and FOV vs depth
%               2.5 26.05.97 SH  -- " -- also for linear arrays
%                   15.09.06 SH  added fix() in argument to avoid error
%                                outputs "assumed kerf" instead of "kerf"
%                   17.08.07 SH  added figure(plotfig) prior to each
%                                subplot call
%
% Called by   : usimcnf
% Calling     : datetext, press
%----------------------------------------------------------------

function curved(transducer, excitation, media, flagg, plotfig,plotfigOrigHandle ) 
global width limit


if flagg(2) ~= 1,
  	disp('Analysis only applicable to curved and linear arrays!');
	return
else
  	cleanscr(plotfig,plotfigOrigHandle)
  	figure(plotfig)
 	subplot(1, 1, 1)

	f      = excitation(1);
	lambda = media(1,2)/f;
	ROC    = -transducer(17);

	if (ROC < -eps)
		disp('Unpredictable results for concave arrays!')
	end

	la     = abs(ROC) <= eps | isinf(ROC); % linear array

%
% In ultrasim chord is stored in transducer(1)
%

	chord   = transducer(1);
	if la
		arc = chord;
	else
		thetamax = 2*asin(chord/(2*ROC));
		arc = thetamax * ROC; 
	end

	FNin = 0.1:0.01:5;

	if la, 
		Range = .01;
	else
		Range = .01:.01:.1; % m
		disp('f/stop defined as range/chord')
	end
%
% R - Range
%
	for R = Range,

%
% A - Aperture; A = R/FN
%
		A = R./FNin;
%
% limit A to maximum extent of array
%
		A = A(find(A <= chord));
		FN = R./A;
		if ~la,

%
% angle that aperture spans in curved array
%
			theta = 2*asin(A/(2*ROC));  % A = chord 
%
% ch1: chord of half aperture
%
			ch1 = A/2;                 % A = chord
%
% b: distance from outermost element to target
%
%			b = sqrt(R.^2 + ch1.^2 + 2*R.*ch1.*sin(theta/2));
%
% b corrected 20.8.96 by S. Bergstoel:
%
                        b = sqrt(R.^2 + (ROC*(1-cos(theta/2))).^2 + ...
			    ch1.^2 + 2*R*ROC*(1-cos(theta/2)));
%
% phi: angle of incidence of outermost element 
%
			phi=(180/pi)*acos((R.^2 + 2*R*ROC-b.^2)./(2*ROC*b)); 
		else
			phi = (180/pi)*atan(A./(2*R));
        end
        
% added 17.08.2007:
        figure(plotfig)
		subplot(1,2,1)
		plot(FN,phi)
		if (length(FN)>0) & ~la, 
			text(FN(length(FN)), phi(length(phi)),[num2str(100*R),' cm'])
		end
		hold on
	end % for Range

	if ~la,
        	title(['CLA, ROC ',num2str(1000*ROC),' mm, FOV ', ...
                num2str(thetamax*180/pi),' deg, f ',num2str(f*1e-6),' MHz'])
		xlabel('f-number, depth as parameter')
	else
 		title(['FLA, Aperture ',num2str(1000*chord),' mm, f ',num2str(f*1e-6),' MHz'])
		xlabel('f-number')
	end
	ylabel('Angle of incidence for outermost element')

%
% add a line for estimated beamwidth of element. 
% Beamwidth is half*distance from peak to first zero
%
% element size: d (relative to lambda)
%
	kerf = 25e-6;
	width = (arc/transducer(2) - kerf)/lambda;
	disp(['Element size is arc/#, assumed kerf ',num2str(kerf*1e6),' microns'])
	limit = .50119; % -6 dB def for beamwidth
	disp(['Beamwidth is ',num2str(20*log10(limit)),' dB point for avg. of rigid and soft baffle'])

        theta_el = fzero('press',pi/6);

%	disp('Element size is arc/#, no kerf, beamwidth is 50% of distance to first zero')
%	if (width < 1),
%		theta_el = pi/4; % due to cosine weighting
%	else
%		theta_el = 0.5*asin(1/d);
%	end
	

	plot(FNin, (180/pi)*theta_el*ones(size(FNin)),'--')
    %
    % added fix() 14.09.2006:
    %
	text(FNin(fix(length(FNin)*0.3)), (180/pi)*theta_el*1.1,[num2str(width),' lambda'])	
	grid
	hold off

	theta_el =  vinput('max angle of incidence for element', ...
	           0, 90, theta_el*180/pi)*pi/180;
% 


        if ~la,
%
% Find f-stop as a function of depth for the given element maximum angle
%
% Increment theta up to max angle:
%
           theta = .01:.01:thetamax;
	   chord = 2*ROC*sin(theta/2);
	   arc   = theta * ROC;
%
% solve first for omnidirectional element (theta_el = pi/2)
%
           Rprime = ROC*(1-cos(theta/2))./cos(theta/2);
	   d = (Rprime + ROC).*sin(theta/2);
%
% then solve for additional R due to theta_el <> pi/2
%

           a = d*sin(-theta_el+pi/2)./sin(theta_el-theta/2);
	   anegative = find(a < 0);
	   a(anegative) = Inf*ones(size(anegative));
	   R = Rprime + a;
	   rlimit = ceil(0.6e7/f)/10;
	   set = find(R< rlimit);
%
% FOV as a function of depth:
%
    % added 17.08.2007:
           figure(plotfig)
           subplot(2,2,2)
	   plot(R(set)*100,theta(set)*180/pi)
	   grid
	   xlabel('Range [cm]') 
	   ylabel(['Max incidence: ',num2str(theta_el*180/pi),' deg'])
	   title('Field of view vs depth')

	% added 17.08.2007:
        figure(plotfig)
	   subplot(2,2,4)
	   plot(R(set)*100, R(set)./arc(set),'-.', R(set)*100, R(set)./chord(set))
	   grid
	   xlabel('Range [cm]') 
	   ylabel(datetime)
	   title('f-number (-), arc/range(-.)')
	   
	 else % linear array
	   rlimit = ceil(0.6e7/f)/10;
	   R = .001:.001:rlimit;   
	   aperture = min(2*R*tan(theta_el), chord);

       % added 17.08.2007:
        figure(plotfig)
	   subplot(2,2,2)
	   plot(R*100,aperture*1000)
	   grid
	   xlabel('Range [cm]') 
	   ylabel(['Max incidence: ',num2str(theta_el*180/pi),' deg'])
	   title('Aperture [mm] vs depth')

       % added 17.08.2007:
        figure(plotfig)
	   subplot(2,2,4)
	   plot(R*100, R./aperture)
	   grid
	   xlabel('Range [cm]') 
	   ylabel(datetime)
	   title('f-number vs depth')
	 end

end

return

