%------------------------------------------------------------------------
% Routine         : beampat.m
%
% Description : Calculates the peak value or the energy of the signal depending
%               on the value of bpflag.
%
% Language    : Matlab 4.1
% Written by  : Sverre Holm VMS, Tor Arne Reinen, SINTEF-DELAB April 1993  
%
% Version no. : 2.0 SH/TAR 
%             : 3.0 FT August-93. 
%               3.1 FT 27.09.93 Phase factor changed in order to get
%                               0 Dc-level of pulse.
%               3.2 FT 13.10.93 Added propagation times for
%                               inhomegeneous media.
%               3.3 FT 06.01.94 Added const phase_add 
%                               Change in call to userpuls 
%               3.4 FT 15.02.94 unabled when flagg(3) is 2. 
%               3.5 FT 21.04.94 Changed call to proptime (_r0)
%               3.6 FT 28.09.94 Changed call to t_inhom and pulse
%               3.7 EH 10.03.95 Removed/modified redundant displayed messages.
%                   EH 16.03.95 Changed direction of time in array_resp and 
%                               commented away mod. of tmax for layer3D
%                   EH 22.03.95 Corrected time argument of phase factor in 
%                               calculation of array_resp and changed sign of
%                               pulse  
%               3.8 SH 11.04.95 Corrected displayed text, removed unused code
%               3.9 LO  8.10.95 tstep mod from 1/4f to 1/6f
%              3.10 LO 10.10.95 transducer included in call to t_inhom
%              3.11 LO 10.10.95 normalization of beam profile turned off
%              3.12 LO 11.10.95 Attenuation messages moved to t_inhom 
%              3.13 LO 13.12.95 Attenuation normalized to the case with 
%                               observation depth "rref" and 
%                               transducer area "pi*(Dref/2)^2", where
%                               rref=0.1 and Dref=0.017;
%              3.14 SH 23.01.98 Removed unmatched end (Matlab 5)
%                      27.09.2006 SH renamed quantize.m to quantizu.m to avoid
%                       conflict with toolbox/simulink/blocks/quantize.m
%                      10.10.06 initalized array_resp=0 for CW/energy case
%
%
% Called by   : bpusim
% Calling     : xyz2rhp , quantizu, xyz2fpts, proptime, quanttxt
%               pulse, abstxt, weightxt, akser, focustxt, t_inhom
% ----------------------------------------------------------------

function [array_resp]=...
beampat(transducer,excitation,media,observasjon,flagg,x,y,z,t,...
        elem_pts,config,option,exact_rx,apod3,bpflag,pvec,plength);

figure(config);
% CW or PW:
CW = (~finite(excitation(9)) | excitation(9) == 0) & excitation(13)==0; 



%                      INFORMATION
%------------------------------------------------------------------------
disp('  ')
disp('****************** beampat.m *********************')
disp('--------------------------------------------------')
if bpflag ==1,     % energy calculations
   if CW 
       disp('Energy of CW is infinite. Please use option "Peak"')
       array_resp = 0; % added 10 oct 2006
       return
   else
     if flagg(3)==1
       disp(['Program for finding the energy beampattern']);
       disp(['for an array in a homogenous medium.']); 
     elseif flagg(3)==3
       disp(['Program for finding the energy beampattern']);
       disp(['for an array in a layered medium.']); 
     end    
   end
else               % peak calculations
   if flagg(3)==1
       disp(['Program for finding the peak beampattern']);
       disp(['for an array in a homogenous medium.']); 
   elseif flagg(3)==3
       disp(['Program for finding the peak beampattern']);
       disp(['for an array in a layered medium.']); 
   end    
 end
disp('  ')



%                  DEFINITION OF PARAMETERS
%------------------------------------------------------------------------
% input : excitation,
fid = 1;


%       LOCAL FLAGS

manual    = 0;                  % manual entry of field points
sar       = 0;                  % synthetic aperture
normalize = 0;                  % normalize distances to fixed focus
show      = 0;                  % show plots of apodization, delays etc


%       EXCITATION

Weig_numb = excitation(10);
pflag     = excitation(13);
f         = excitation(1);
Fs        = excitation(11);
intfact   = excitation(5);

if flagg(4) == 1 | flagg(4) == 3
  %convert from rectangular coordinates
  [ElF_r, ElF_theta, ElF_phi] = xyz2rhp(excitation(2),...
  excitation(3),excitation(4));
end


%       MEDIUM

c = media(1,2);


%       DEDUCED PARAMETERS

%pflag = 1 if a user-defined pulse is to be used in the calculations

if pflag == 0, 
   plen = excitation(9)/excitation(1);
else
   plen = plength;
end

N=length(elem_pts(1,:));


%
%	MICRO STEERING PARAMETERS
%

if flagg(1) == 3,
   psi    = excitation(22);			% micro-steering angle
   no_sub = excitation(23);			% # of sub apertures
   M      = input('Length of sub aperture : ');	% size of sub-array
   pitch  = transducer(1)/(transducer(2))/(c/f);
end


%
%                 QUANTIZATION OF DELAYS
%----------------------------------------------------------------------
% quantize and plot the time delays to accuracy qt
% (only if interpolation is off)
% input : exact_rx, excitation 

[delta_t_rx] = quantizu(excitation,exact_rx);
if show==1      % PLOTTING
  subplot(3,3,7)
  plot((0:N-1),(exact_rx)*1e6)
  grid
  title(['Exact time delay']);
  ylabel('microseconds')
  xlabel('Array element')
  %
  subplot(3,3,8),
  plot(delta_t_rx*1e6),
  title(['Quantized time delay']);
  xlabel('Array element')
  %
  %plot delay quantization error
  %
  subplot(3,3,9)
  plot((0:N-1),(exact_rx-delta_t_rx)*Fs*intfact)
  axis([0, N-1, -0.5, 0.5]);
  grid
  title(['Time delay quantization error']);
  ylabel(['Rel. quantization error, ',quanttxt(excitation)])
  xlabel('Array element')
  axis('normal');
end



%          OBSERVATION POINTS ( alias sources alias field_pts )
%---------------------------------------------------------------------------
% input  : x,y,z,t,observasjon,ElF
% output : fpts_local

if (option(1) == '1')
  %observation along a line, generate field points:
  [fpts_local] = xyz2fpts(x,y,z);
else
  disp('Observations other than along a line is not implemented!')
  return 
end



%  CALCULATION (and plot) OF PROPAGATION-TIME/DISTANCE and ATTENUATION
%---------------------------------------------------------------------------
% input  : delta_t_rx, ElF_r, fpts_local, elem_pts, c, normalize, z  
% output : delta_t_source, distance

near_field = finite(z(1));
n_fpts_local = length(fpts_local(1,:));

if flagg(3)==1,
  [delta_t_source, distance] = proptime(fpts_local,...
  elem_pts,c,near_field,CW);

elseif flagg(3)==3,

  [delta_t_source, distance,attenuation] = ...
  t_inhom(transducer,excitation,media,x,y,z,elem_pts,flagg);  
elseif flagg(3)==2
  disp(' Beampattern works only with media flag set to ');
  disp(' ''homogeneous'' or ''3D layered''');
  return
end


% Calculation of attenuation in a homogeneous medium:
if flagg(3)==1,
  attenuation=ones(size(distance));

  % weighting each ray with its corresponding transducer areal 
  if flagg(2)==3  %annular transducer
    Dtrans=transducer(1);
    ntranspoint=size(elem_pts,2);
    arealperray=pi*(Dtrans/2)^2/ntranspoint;
  elseif flagg(2)==1 % 1D og 2D rektangular
    ntranspoint=size(elem_pts,2);
    arealperray=transducer(1)*transducer(5)/ntranspoint;
  else
    disp('Warning in beampat: area of transducer unknown')
    arealperray=1;
  end
  attenuation=attenuation*arealperray;

  % Absorption
  alph=media(1,9);
  bet=media(1,10);
  f=excitation(1);
  intatt = exp(-alph*((f/1e6)^bet)*distance);
  attenuation=attenuation.*sqrt(intatt);
end


% Attenuation standarized to the case with
% observation depth "rref" and transducer area "pi*(Dref/2)^2":
if (flagg(3)==1)|(flagg(3)==3)
  rref=0.1;
  Dref=0.017;
  attenuation=attenuation*rref/(pi*(Dref/2)^2);
end


if  near_field & show==1,       % PLOTTING
  subplot(3,3,3),
  plot(1000*distance'),
  title('distance in mm for each array element to each field point'),
  grid
end


if (min(min(distance)) <= 0 & flagg(3)==1) | min(min(distance))<-1e-10
  disp('Transducer - observation distances <= 0 are not allowed!')
  return
end


delta_t_source =  delta_t_source - ones(n_fpts_local,1)*delta_t_rx;


%             CALCULATION OF PHASE_FACT
%----------------------------------------------------------------------  
if (sar ~= 1)
  phase_fact = j*2*pi*f;
  phase_add = -j*pi/2;
else
  phase_fact = -j*4*pi*f;
end

if pflag ~= 0
  phase_fact = 0;   % The phase is accounted for in the pulse-vector
  phase_add = 0;
end



%          CALCULATION OF ARRAY-RESPONS
%          Finding start-/stop-time and step
%----------------------------------------------------------------------
if bpflag ~= 0 & ~CW,  
  tmax = -min(min(delta_t_source)) + max(plen)/2;
  tmin = -max(max(delta_t_source)) - max(plen)/2;
  %if flagg(3)==3
  %  [ind1 ind2] = find((delta_t_source./distance)==...
  %  max(max(delta_t_source./distance)));
  %  tmax = delta_t_source(ind1(1),ind2(1)) + max(plen)/2;
  %end
  if pflag~=0
    % Times refer to the start of the pulse instead of the middle.
    tmin = tmin + max(plen)/2;
    tmax = tmax + max(plen)/2;
  end
  tstep = 1/(6*f); 
  samples = fix((tmax-tmin)/tstep) + 1;
  array_resp = zeros(samples,n_fpts_local);

  fprintf(fid,['...WAIT Calculating ...       percent    ']);
  pros = 0;
  fprintf(fid,'\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b %6.2f percent',pros);
else
  tmin = 0;
  tmax = 0;
  tstep = 1;
  array_resp = zeros(1,n_fpts_local);
end
count = 0;

%**********************************************************************
%          CALCULATION OF ARRAY-RESPONS
%          Main Loop - Samples taken at 6*f
%----------------------------------------------------------------------

for t=tmin:tstep:tmax
  count = count + 1;
  %          CALCULATION OF SHAPE
  %----------------------------------------------------------------------
  if pflag ~=0,
    [shape]=userpuls(elem_pts,(delta_t_source+t),pvec,plen,attenuation); 
  else
    [shape,CGp]=pulse((delta_t_source+t),plen,Weig_numb,attenuation);
  end
  %
  %(in the focused case, each element has different beam pattern due to 
  %different local angle of incidence over the array, this is not taken into 
  %account here) 


  %	   CALCULATION OF MICRO STEERING DELAY AND APODIZATION
  %----------------------------------------------------------------------

  if flagg(1) == 3,

    apod_numb=excitation(5);
    if apod_numb     == 0,
       apod_weight = boxcar(M); 
    elseif apod_numb == 1,
       apod_weight = hamming(M); 
    elseif apod_numb == 2,
       apod_weight = hanning(M);
    elseif apod_numb == 3,
       apod_weight = kaiser(M,.1102*(82-8.7));
    end
 
    if apod_numb ~= 4,
        disp('WARNING - APODIZATION SHOULD BE USER DEFINED');
        elem_apo     = apod_weight;
        apod_sub_el  = M*elem_apo'/sum(abs(elem_apo));
        for cnt2=1:no_sub,
           bef = (cnt2-1)*(N-M)/(no_sub-1);
           aft = N-M-bef;
           ap(cnt2,:) = [zeros(1,bef) apod_sub_el zeros(1,aft)];
        end

    elseif apod_numb == 4,

       if rem(no_sub,2) ~= 0,
          disp('FATAL ERROR - ONLY TWO SUBAPERTURES IMPLEMENTED');
          return;
            
       else 
          if no_sub ~= 2,
             disp('FATAL ERROR - ONLY TWO SUBAPERTURES IMPLEMENTED');
             return;
          else
             apod_weight = hanning(4*M-2*N);
             apod_weight = [apod_weight(1:M-N/2);zeros(N/2,1)];
             k = kaiser(N,7);
             apod_weight(2*M-N+1:M) = k(M+1:N);
             apod_weight(M-N/2+1:2*M-N) = k(N/2+1:M) - apod_weight(M-N/2:-1:1);
             ap(1,:) = [apod_weight(M:-1:1)' zeros(1,N-M)];
             ap(2,:) = [zeros(1,N-M) apod_weight'];      
             x  = linspace(1,M,M);
             m0 = sum(apod_weight)/M;
             m1 = x*apod_weight/M/m0;
             m2 = x.^2*apod_weight/M/m0-m1^2;
             m0 = 1;
             dM = (N-M);
             phase = 2*pi*pitch*psi*dM*(1+4*m2/(m0*dM^2));
             fprintf(fid,'Using phase correction: %6.2f [deg]',phase*180/pi);

          end

       end

    end


    %   CALCULATION OF SUB-APERTURE APODIZATION
    %	AND ARRAY RESPONSE
    %----------------------------------------------------------------------

    array_resp = zeros(1,n_fpts_local);

    for cnt = 1:no_sub,
       apod3 = ap(cnt,:);
       if near_field,
          array_resp = array_resp-exp((1-cnt)*j*phase)*apod3*(shape...
          .*(exp(phase_fact * (delta_t_source + t) + phase_add))./(distance))';
       else
          array_resp = array_resp-exp((1-cnt)*i*phase)*apod3*(shape...
          .* exp(phase_fact * (delta_t_source + t) + phase_add))';
       end
    end



    %
    %	VERIFY CORRECT ATTENUAION
    %---------------------------------------------------------------------

    attn_msr = 0.014278/max(abs(array_resp));
    attn_the = m0/(m0-2*pi^2*pitch^2*psi^2*m2*(1+4*m2/(m0*dM^2)));
    fprintf(fid,'\nTheoretical attenuation (dB) : %f',20*log10(attn_the));
    fprintf(fid,'\nMeasured    attenuation (dB) : %f',20*log10(attn_msr));



  %        CALCULATION OF ARRAY-RESPONSE
  %       Finding field value at sample-time
  %----------------------------------------------------------------------
  % input : apod3, shape, phase_fact, delta_t_source, distance, CW, nearfield
  % output: rms
  %
%figure(3);
%tmp = array_resp;
%tmp = abs(tmp).^2;
%tmp = tmp/max(tmp);
%plot(linspace(-200,200,length(tmp)),10*log10(tmp));pause
  elseif ~CW

    if near_field,
      array_resp(count,:) = -(apod3)*(shape...
         .*(exp(phase_fact * (delta_t_source + t) + phase_add))./distance)';
   else                                   
      array_resp(count,:) = -apod3*(shape...
         .*exp(phase_fact * (delta_t_source+t) + phase_add))';
    end
  else
    if near_field,
      array_resp = -apod3*...
      (exp(phase_fact * delta_t_source + phase_add)./distance)';
    else
      array_resp = -apod3*(exp(phase_fact * delta_t_source + phase_add))';
    end
  end
 
  if bpflag ~=0,
    pros = count/(samples)*100;
    fprintf(fid,'\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b %6.2f percent',pros);
  end
end
