%------------------------------------------------------------------------
% Routine     : focrect()
%
% Description : Calculate focusing delays for centers(1:3,number_of_points),
%	        relative to origin, when the sound velocity is c, and
%               focus is given in spherical coordinates: (r,theta,phi) 
%               with the angles in degrees. Unfocused delays, i.e. focus
%               at r = infinity, is obtained by setting r = 0. 
%               The components of each elem_pts - point are (x,y,z).
%
% Language    : Matlab V4.0
% Written by  : Tor Arne Reinen, SINTEF-DELAB April 1992  
%
% Version no. : 1.0 
% Version no. : 1.1 SH  14.04.93                                     
% Version no. : 1.2 VEB 22.04.93                                    
% Version no. : 1.3 TKL 30.07.93 Changed some names
% Version no. : 1.4 SH   3.12.93 Completely rewritten:
%				 Corrected errors and speeded up 
%				 Added transducer in call
% Version no. : 1.5 JOE 16.04.94 No calculation for thinned elements
%               1.6 FT  25.04.94 Removed last command, ls
%               2.2 SH  04.06.96 centers is input to focrect: 2D focusing
%               2.3 SH  21.01.97 Axicon focusing, excitation is input par,
%		2.4 NAA 16.04.97 Hybrid focusing
%                    SH 26.06.97 Hybrid focusing also for range = Inf
%
% Called by   : focus
% Revised     : 
%------------------------------------------------------------------------

function [delays,ctr] = focrect(excitation,c,elem_pts,centers,transducer)

     [m,N] = size(elem_pts);
     r = excitation(2);
     theta = excitation(3);
     phi = excitation(4);
     
%
% translate element centers to find phase center of each point
%

 	ctr = zeros(3, N);
	for k = 1:N
		naz = elem_pts(4,k);
			if (naz > 0)
 				el = elem_pts(5,k);
				ind = find(centers(4,:) == naz & centers(5,:) == el);
				ctr(1:3,k) = centers(1:3,ind); 
			end
	end
	
	if (length(excitation) < 25) excitation(24:25) = [0 0]; end
	



    if (excitation(24) == 2)
%
% hybrid focusing (axicon and spherical)
%
      alpha = .5;      % mixture ratio

      ax_ang = excitation(25);
      for k= 1:N
	x = ctr(1,k); z = ctr(3,k);
	if (x >= 0) 			% positive x
	  delays(k) = (z-x*tan(ax_ang + phi))/c;
	else
	  delays(k) = (z-x*tan(-ax_ang + phi))/c;
	end
      end	
    
      if (r~=0 & finite(r))
	foc_pt = r*[sin(phi)*cos(theta)
		    sin(phi)*sin(theta)
		    cos(phi)];
 	dr3 = ctr - foc_pt*ones(1,N);
	r_el3 = sqrt(sum(dr3.*dr3));
	delays2 = (r - r_el3)/c;
      else
	disp('Warning! Hybrid focusing for r=Inf is not fully tested!')
	foc_pt = [sin(phi)*cos(theta)
	    sin(phi)*sin(theta)
	    cos(phi)];
	delays2 = foc_pt'*ctr/c;
      end
      delays = alpha*delays + (1-alpha)*delays2;
      
    elseif (excitation(24) == 1),
%
% conical focusing (axicon)
%
        ax_ang = excitation(25);
	for k= 1:N
	  x = ctr(1,k); z = ctr(3,k);
	  if (x >= 0) 			% positive x
	    delays(k) = (z-x*tan(ax_ang + phi))/c;
	  else
	    delays(k) = (z-x*tan(-ax_ang + phi))/c;
	  end
	end	

      else
%
% compute exact steering- and focusing delays for spherical focusing
%
      if r~=0 & finite(r)
	foc_pt = r*[sin(phi)*cos(theta)
	    sin(phi)*sin(theta)
	    cos(phi)];
	dr3 = ctr - foc_pt*ones(1,N);
	r_el3 = sqrt(sum(dr3.*dr3));
	delays = (r - r_el3)/c;
      else
	foc_pt = [sin(phi)*cos(theta)
	    sin(phi)*sin(theta)
	    cos(phi)];
	delays = foc_pt'*ctr/c;
      end
    end