%------------------------------------------------------------------------
% Routine     : genremez.m
% Description : A general Remez exchange algorithm
% Language    : Matlab 4.2
% Written by  : Jan Ove Erstad , IfI-UiO
%
% Version no. : 1.0 JOE 15.10.94 First version
%               1.1 JOE 15.11.94 Name changed to genremez
%               1.2 BJE 25.04.95 removed ConfigOrigHandle from functioncall
%
% Called by   : usimcnf
% Calling     : none
%------------------------------------------------------------------------
function [h] = genremez(lgrid,elem_pts,transducer,option,media,excitation,flagg,config)
%
  
  if any(elem_pts(2,:) ~= 0) 		%check if array is linear
      disp('Optimal approximations designed for linear arrays only.');
      return
  end
  
% INITIALIZE
  fid    =  1;
  el_p   =  elem_pts(:,elem_pts(4,:) >= 0 ); % non-thinned elements
  nfilt  =  max(size(el_p)); 		% array length
  cmptr  =  computer;
  lambda =  media(1,2)/excitation(1); 	% wavelength
  xi     =  2*el_p(1,(el_p(1,:) >= 0))./lambda; %normalized wavenumber
  
% CHECK IF ARRAY FULFILLS SOME CHEBYSHEV DEMANDS
  if max(size(elem_pts(1,elem_pts(1,:) < 0))) ==...
          max(size(elem_pts(1,elem_pts(1,:) > 0))) % check if centered
      neg_elem = fliplr(el_p(1,el_p(1,:) < 0));
% check if equal number of nonthinned elements
      if max(size(neg_elem)) == max(size(el_p(1,el_p(1,:) > 0)))
% check if symmetric elements
          if any(abs(abs(neg_elem) - el_p(1,el_p(1,:) > 0)) > eps) 
              disp('ERROR! The array is not symmetric !');
              return
          end
        else
          disp('Number of nonthinned elements on each side of origo is not equal!');
          return
      end
    else
      disp('ERROR! The array is not centered around x=0!');
      return
  end 					%if centered
  
  clc
  disp(' ');  
  disp(' Generalized Remez algorithm ');
  disp('-----------------------------');
  fprintf(fid,['Grid spacing value:         ',num2str(lgrid),'.\n']);
%
%USER INPUT
%
  disp('Input format is : Pasband cut-angle , stopband cut-ang. , ');
  disp(' optimization limit angle and weight difference factor K.');
  angl = input('Give angles (degrees) and K (stopband): ');
  if isempty(angl)
      disp('Unknown command!');
      disp('Input format is cut-angle i degrees.');     
    else
      if (angl(1) <= angl(2)) & (angl(2) < angl(3))
          disp(' ');  
          disp(' REMEMBER to point mouse on Config Window when giving commands!');
          disp(' ');  
          ff = [radians(angl(1:3))]; 		% normalized frequency
          K  = angl(4); 			% band weighting
        else
          error('Wrong input parameters, angles not in increasing value!');
      end
  end
  
  ff  = pi*sin([0 ff]);		% cut-off angles/frequencies 
  aa  = [1 1 0 0]; 			% desired amplitude
  wtx = [1 K]; 				% weighting of errors
  
  if nfilt < 4 				%check if array is shorter than minimum
      error('Number of elements must be at least 3!');
  end
  
%check if even or odd numbered array
  nodd	=  rem(nfilt,2); 		% nodd == 0 -> filter length is even
% nodd == 1 -> filter length is odd
  nfcns	=  fix(nfilt/2);                % number of basis functions
  if nodd == 1 
      nfcns = nfcns + 1; 			% odd case -number of basisfunctions 
  end
  
% SET UP THE DENSE GRID (the grid depends on the cut angles)
  edge     = ff;
  delf	 = pi/(lgrid*nfcns); 		% step size
  dgrid(1) = edge(1); 			% zero angle/frequency
  fup      = edge(2); 			% mainlobe edge
  dgrid    = [dgrid (dgrid(1)+delf):delf:(fup+delf)]; % pass band grid
  uppr     = max(size(dgrid)); 		%number of gridelements in passband
  dgrid(uppr-1) = fup; 			% mainlobe edge as last grid position
  sel      = 1:uppr-1;
  des(sel) = polyval(aa(1),dgrid(sel)); 	% desired beampattern in pass band
  wt(sel)  = wtx(1)./ones(size(sel)); 	% weight function in pass band
  
  j    = uppr; 				% first vacant grid position
  dgrid(uppr)= edge(3); 		% stopband edge
  fup  = edge(4); 			% upper edge angle/frequency (90 deg)
  dgrid= [dgrid (dgrid(j)+delf):delf:(fup+delf)]; % stop band grid (+delf)
  uppr = max(size(dgrid)); 		% grid vector size
  dgrid(uppr-1) = fup; 			% end angle at last grid position
  sel  = j:uppr-1;
  des(sel)= polyval(aa(3),dgrid(sel)); 	% desired beampattern in stop band
  wt(sel) = wtx(2)./ones(size(sel)); 	% weight function in stop band
  
  ngrid   = uppr  - 1; 			% number of grid elements
  if nodd == 0 & dgrid(ngrid) > pi-delf
      ngrid = ngrid -1;
  end
  
% INITIAL GUESS FOR THE EXTREMAL FREQUENCIES 
  temp = (ngrid - 1)/nfcns; 		% initial extremum step size
  j    = 1:nfcns; 			% basisfunction count
  iext = fix([(j-1)*temp+1 ngrid]); 	% equally spaced extremums
  nm1  = nfcns - 1;
  nz   = nfcns + 1; 			% number of extremums
  
%
%THE REMEZ EXCHANGE ALGORITHM
  itrmax	=  20;
  niter	=  0;
  jchnge	=  1;
  
% outer exchange loop
  while jchnge > 0
      
%    iext(nzz)	=  ngrid + 1;
      niter	=  niter + 1;
      if niter > itrmax
          break
      end
      
% SYSTEM 
      ll    =  iext(1:nz); 			% extremum grid coordinates
      Nf    =  length(ll); 			% number of extremums
      Ae    =  zeros(length(xi)+1,Nf); 	% initialize system matrix
      for k=1:Nf
          Ae(k,:)  =  [cos(xi.*dgrid(ll(k))) ((-1).^(k-1))./wt(ll(k))];
      end
      
      condition_number= cond(Ae);
      determinant     = det(Ae);
      D = des(ll)'; 			%'
      
% Calculate coefficients
      alpha = Ae\D; 	  % solving the system
      al = alpha;
      delta = alpha(max(length(alpha))); 	% the deviation
      alpha = alpha(1:max(length(alpha)-1))'; %'the weight parameters
      dev= 20*log10(delta) + 20*log10(1/K);
% CALCULATING THE POLYNOMIAL/BEAMPATTERN FOR POSITIVE ANGLES    
      if  nodd == 1
          fr  = dgrid;
        else
          fr  = dgrid(1:ngrid);
      end
      
      for lk=1:max(size(fr))
          cosfu  =  [cos(xi.*fr(lk))]; 	% the basis functions
          FR(lk) = sum(alpha.*cosfu);
      end
      
      rounderr = find(abs(FR) < 1e-11);
      if ~isempty(rounderr)
          FR(rounderr) = zeros(1,length(rounderr));
      end
      
% FINDING THE EXTREMAL ANGLES/FREQUENCIES
% CALCULATE THE DERIVATIVE
      indx     = 1:length(FR)-1; 		%estimating with formula 1
      dPw      = [0 (FR(indx+1)-FR(indx))./(fr(indx+1)-fr(indx))];
      dPchnge  = dPw(indx).*dPw(indx+1); 
      dPsign   = find(dPchnge < 0);		%find the points with sign changes 
      
      indxx     = 2:length(FR); 		%estimating with formula 2
      FRR = [0 FR];
      dPww      = [0 (FRR(indxx+1)-FRR(indxx-1))./(2*(fr(indxx)-fr(indxx-1)))];
      dPchngee  = dPw(indxx-1).*dPw(indxx); 
      dPsignn   = find(dPchngee < 0);	%find the points with sign changes 
      
      findcut  = find(fr == ff(2) | fr == ff(3));
      xtr  = dPsign(find(dPsign > findcut(2))); 
      iext = [findcut xtr]; 		% New extremums
%  Check number of extremums
      omitd = 0;
      if max(size(iext)) > nz
          disp('Total number of extremums is larger than maximum.');
          tooclose = find((xtr - findcut(2)) < 3);
          if ~isempty(tooclose)
              xtr(tooclose) = [];
          end
          while max(size(xtr)) > nz - 2
              leasterr = min(abs(abs(FR(xtr)) - des(xtr)));
              xtr = xtr(find(abs(FR(xtr)) ~= leasterr));
              omitd = omitd + 1;
          end
          fprintf(fid,[num2str(omitd),...
              ' extremums omitted in this iteration','.\n']);  
          disp(' ');
          iext = [findcut xtr]; 		% New extremums
%    keyboard
      end  
      if max(size(iext)) < nz
          disp('Too few extremums!');
          iext = [iext ngrid];
      end
      
      
% VISUALIZE THE ALGORITHM IN EACH STEP
      subplot(1,1,1);
      if nodd == 1
          angaxis = pi*(0:.0025:1);
        else
          angaxis = pi*(0:.0025:1-0.0025);
      end
      for lk=1:max(size(angaxis))
          cosfu  =  [cos(xi.*angaxis(lk))]; 	% the basis functions
          Resp(lk) = sum(alpha.*cosfu);
      end
      
% check deviation
%lores = find(Resp(angaxis <= ff(2)));
%maxdev= 20*log10(abs(max(Resp(lores)-1))/K)
      
      plot(angaxis,20*log10(abs(Resp)/abs(Resp(1))),'r'); grid on 
      grid on 
      title('COSINE-POLYNOMIAL');
      xlabel('pi*sin(phi)');
      ylabel('Response [dB]');
      hold on
% plot initial cut angles
      temp=axis;
      plot([fr(findcut(1)) fr(findcut(1))],[temp(3) temp(4)],'g:')
      plot([fr(findcut(2)) fr(findcut(2))],[temp(3) temp(4)],'g:')
      
      fre=fr(ll); 			%Extremal frequencies of the previous step
      for km=1:max(size(ll))
          frr(km)= sum(alpha.*cos(xi.*fre(km)));
      end
      plot(fre,20*log10(abs(frr)/abs(FR(1))),'cx');
      
      fl=fr(iext); 				% Extremal frequencies of this step
      for kl=1:max(size(fl))
          flr(kl)= sum(alpha.*cos(xi.*fl(kl)));
      end
      plot(fl,20*log10(abs(flr)/abs(FR(1))),'yo');
      hold off
      
      if max(size(iext)) ~= max(size(ll))
          disp(' ');
          disp('Error, the algorithm has not succeeded in finding the ');
          disp('correct number of  extrema! Badly conditioned matrix?');
        else
          diffext = find(iext - ll ~= 0); 	%find extremum changes
          if ~isempty(diffext)
              fprintf(fid,[' ',num2str(length(diffext)),...
                  ' changes of extremum frequencies in iteration  ',...
                  num2str(niter),'.','  Push a key!','\r']);
              [ss,sss,but]= ginput(1);
              if (but == 120) | (but == 113)
                  jchnge = 0;
              end
            else
              disp(' ');
              fprintf(fid,[' No change of extrema in iteration  ',...
                  num2str(niter),'.','\r']);
              disp(' ');
              disp('Push a key to quit.');
              ginput(1);
              jchnge = 0;
          end
      end
  end 					% END OUTER EXCHANGE LOOP
  
% Write zero crossing and sidelobe maximum
  fprintf(fid,['Solution arrived in ',num2str(niter),' iterations','.\n']);
% find the first nil
  dev   = max(FR(iext(3):max(size(FR))));
  indx  = find(abs(abs(FR) - dev) < 1e-10);
  pchng = min(indx);
  fprintf(fid,['Max. sidelobe at ',num2str(180/pi*asin(fr(pchng)/pi)),...
      ' deg.','\n']);
  indx  = iext(2):iext(3);
  first = FR(indx(1));
  pchng = min(find(first*FR(indx) < 0));
  fprintf(fid,['Initial cut angle : ' ...
      num2str(180/pi*asin(fr(findcut(2))/pi)) ...
      ' degrees at sidelobe level ' ...
      num2str(20*log10(FR(findcut(2))/abs(FR(1)))) 'dB' '\n']);
  fprintf(fid,['First zero-crossing at :  ',num2str(180/pi*asin(fr(indx(pchng-1))/pi)),...
      ' deg.','\n']);
%  maximum sidelobe level
  slevel = 20*log10((dev)/abs(FR(1)));
  fprintf(fid,['Max. sidelobe level:  ',num2str(slevel),' dB.','\n']);
  
% CALCULATE THE IMPULSE RESPONSE
  if nodd ~= 0 				%inversion formula in odd length case
      h = [0.5*alpha(nz-1:-1:2) alpha(1)];
    else 					%inversion formula in even length case
      h = 0.5*fliplr(alpha);
  end
  
% MAKING THE APODIZATION FOR BOTH POSITIVE AND NEGATIVE ELEMENTS
  lenh=max(size(h));
  h = [h(1:lenh-nodd) h(lenh:-1:1)]; 	% making symmetric impulse response
  h = h/max(abs(h));
  
  subplot(2,1,1); 			% plot apodization
  stem(el_p(1,(el_p(4,:) >= 0))./lambda,h); grid
  ylabel('Weighting');
  xlabel('Element Positions in Wavelengths');
  
  subplot(2,1,2); 	% plot polynomial values
  angaxis = pi*(0:.0025:1);
  for lk=1:max(size(angaxis))
      cosfu  =  [cos(xi.*angaxis(lk))]; 	% the basis functions
      P(lk) = sum(alpha.*cosfu);
  end
  
  plot(angaxis/pi,20*log10(abs(P)/abs(P(1))),'r'); grid on 
  xlabel('sin(phi)');
  ylabel('Response [dB]');
  hold on
  axis([0 1 -80 0]);
  disp('FINISHED!');
  disp(' ');
  disp('Push a key to return to optimization menu. ');
  fprintf('\n');
  ginput(1);
%------------------------------------------------------------------------
  
