
%------------------------------------------------------------------------
% Routine         : annular()
%
% Description     : place npts points uniformly on an annulus of radius
%		    radius
%		    if npts = 4*i for some integer i, half the points are
%		    uniformly distributed along each axis. if npts = 4*i + j,
%		    j is truncated to 1, and a point is placed in origo.
%
%	   	    on exit el_pts(1:3,n) contains (x,y,z) coordinates of elem-
%                   nt number n.  the x axis is scanned first , then the y axis, 
%		    both in the positive direction. if origo is included, it is
%		    included in the x scan.
%
% Language        : Matlab 4.0
% Written by      : Sverre Holm, Vingmed Sound  June 1992 / VEB: 20/4-93         % Modified           : ver 1.1 20/4-93 by VEB
%	          :     1.2 16.08.94    SH, output Fresnel number and intensity
%                                           max 
% Version no.     : 1.3 31.01.96 GHJ Bessel Beam
%                   1.4 10.01.96 GHJ more options for Bessel beam
%                   1.5 11.11.96 SH  Changed Bessel options
%                   1.6 27.01.97 SH  Conical transducer (equal area)
%
% Called by       : t3usim
%------------------------------------------------------------------------

function [el_pts] = annular(transducer,excitation,media);

disp('	WAIT CALCULATING ELEMENT-POINTS - ANNULAR')

radius=transducer(1)/2;
	
Npts=transducer(3);
Nelem = transducer(2);
F=transducer(9);

if (transducer(18) == 0 | transducer(18) == 1 | (transducer(18)>= 4 &  transducer(18) <= 10) ),

  %
  %spread points over surface for equal area and equal width
  % and conical
  %
  
  %
  %first estimate grid parameters for hexagonal grid
  %
  dy = sqrt(0.5*sqrt(3)*pi/Npts)*radius; % vertical sample distance
  dx = 2*dy/sqrt(3); 			 % horizontal sample distance
  x_offset = dx/2;
  
  % ------------generate coordinates, x and y, projected into the z=0 plane
  %             start at origin
  
  el_pts = zeros(4,Npts-10);
  n = 1;
  y = 0;
  row = 0;
  while y <= radius,
    if (rem(row,2) == 0),
      x = 0;
    else
      x = x_offset;
    end;
    if (x == 0) & (y ~= 0),
      n = n + 1;
      el_pts(1,n) = x;
      el_pts(2,n) = y;
      
      n = n + 1;
      el_pts(1,n) = x;
      el_pts(2,n) = -y;
      
      x = x + dx;
    end;
    while x^2 + y^2 <= radius^2,
      
      if (x ~= 0);
	n = n + 1;
	
	
	el_pts(1,n) = x;
	el_pts(2,n) = y;
	
	n = n + 1;
	el_pts(1,n) = -x;
	el_pts(2,n) = y;
      end;				
      
      if (y ~= 0),
	n = n + 1;
	el_pts(1,n) = x;
	el_pts(2,n) = -y;
	
	n = n + 1;
	el_pts(1,n) = -x;
	el_pts(2,n) = -y;
      end;
      x = x + dx;
    end;
    y = y + dy;
    row = row + 1;
  end;
  
  trueNpts=n;
  
  
  if (transducer(18) ~= 10);  
    disp('Spherical transducer')
  % Calculating z-coordinate of annulus VEB 22/3-93.
  % This is not a very good solution: The points will no longer be 
  % uniformly distributed on the transducer surface. But it is simple: 
  % one line:
    if (F == 0 | F == Inf | F == -Inf),
      el_pts(3,:) = zeros(size(el_pts(1,:)));
    else
      el_pts(3,:) = sign(F)*(abs(F) - sqrt( F.^2 - (el_pts(1,:).^2 + el_pts(2,:).^2) ));
    end
  % end: VEB 22/3-93
  else % conical
    disp('Conical transducer');
      el_pts(3,:) = sin(transducer(12)) * sqrt(el_pts(1,:).^2 + el_pts(2,:).^2);
  end  
  % Calculation of el_pts(4,:) which is the element (ring) number:
  % VEB 20/4-93
  
  r0=sqrt(el_pts(1,:).^2+el_pts(2,:).^2);
  for n=1:Nelem

  if (transducer(18) == 0 | (transducer(18) >= 4 & transducer(18) <= 10) )
%
% equal area
%
  	  ind=find( r0<radius*sqrt( n/Nelem) &  r0 >= radius*sqrt( (n-1)/Nelem));
elseif (transducer(18) == 1 )
%
% equal width (SH: 29 April 1993)
%
  	  ind=find( r0<radius*( n/Nelem) &  r0 >= radius*( (n-1)/Nelem));
    end
    if isempty(ind)
      disp(' ')
      disp('---------      WARNING:   ----------')
	disp(['No points in element # ',num2str(n)])
      disp('Increase the number of points!')
      disp(' ')
    else
      el_pts(4,ind)=n.*ones(size(ind));
    end
  end

  % end: VEB 20/4-93
  
  disp(['Desired number of elements: ',num2str(Npts),', actual ',num2str(trueNpts)])

%
% SH 29. April 1993: added output of element distance
%
% compute pitch (element distance)
%
	lambda = media(1,2)/excitation(1);
  
disp(' ');
disp('          point distance')    
disp(['   x : ',num2str(1000*dx),' mm, ',num2str(dx/lambda),' lambda']); 
disp(['   y : ',num2str(1000*dy),' mm, ',num2str(dy/lambda),' lambda']); 
%
% 	Fresnel number
%
	S_annular = F * lambda/(radius*radius);
	Int_max = F/(1+S_annular/2);
	disp(' ');
	disp(['   Fresnel number: ',num2str(S_annular)]);
	disp(['Intensity maximum: ',num2str(1000*Int_max),' mm']);
	disp(' ');

else
  
  %
  %distribute all elements around perimeter of circle
  %
  %code taken from circle.m written by T.A. Reinen SINTEF-DELAB
  
  
  start_angle = 0;
  dphi = 2*pi/Nelem;
  sphi = pi/180*start_angle;
  phi = 0:dphi:(2*pi - 0.9*dphi);
  phi = phi + sphi;
  el_pts(1:2,:) = radius*[cos(phi)
      sin(phi)];
  el_pts(3,:) = zeros(1,Nelem);
  el_pts(4,:) = [1:Nelem];
  
  text = sprintf('circular array, radius %g,  %g elements',radius,Nelem);
  disp([' ',text]);
  
end 

disp('	FINISHED')








