%------------------------------------------------------------------------------
% Function    : [nstart,nend,n6dB_1,n6db_2] = 
%                      function findzeru(n6dB_1,n6dB_2,W,points);
%
% Description : Finds first zero point in beam pattern. 
%               Controls the 6db-points and corrects them if required.
%
% Language    : Matlab 4.2
% Written by  : Andreas Austeng
%
% Version no. : 1.0 AA  23.10.96 First version.
%               1.1 SH  04.06.97 Matlab 5: added round() to avoid warning
%                                removed unmatched 'end'
%                   SH  08.03.00 extra test for while to avoid negative index
%                                test for presence of zero
%                   SH  27.09.06 renamed from findzero to findzeru to avoid
%                                conflict with toolbox/ident/idutils/findzero.m
%
% Called by   : resp1d
% Calling     : 
%
%------------------------------------------------------------------------------

  function [nstart,nend,n6dB_1,n6dB_2] = findzeru(n6dB_1,n6dB_2,W,points);
  
  nstart = round(n6dB_1);
  nend = round(n6dB_2);
  nwidth = nend-nstart;
  if (nwidth > length(W)/2) % no zeros!
     return
  end   
  if (abs(W(nstart)) < abs(W(nstart-1)))
    % This is a sidelobe (hopefully the first!), not the mainlobe!
    i = max(nstart,2);
    while (abs(W(i)) < abs(W(i-1)))
      i=i+1;
      if (i > points) % Walking upward, take care!
	if (abs(W(1)) > abs(W(points)))
	  i = 2;
	else
	  i = points;
	  break;
	end
      end
    end % while-loop
    nstart = i;
 else
    % We're hopefully on the mainlobe!
    i = max(nstart,2);
    while (i>1 & (abs(W(i)) > abs(W(i-1))))
      i=i-1; 
      if (i < 1) % Walking downward, take care!
	if (abs(W(points)) > abs(W(1)))
	    i = points - 1;
	  else
	    i = 1;
	    break;
	  end
	end
%     end % extra end, removed 4.6.97 SH
      nstart = i;
    end % while-loop
  end % if-else
  
  if (abs(W(nend)) > abs(W(nend-1)))
    % This is a sidelobe (hopefully the first!), not the mainlobe!
    i = max(nend,2);
    while (abs(W(i)) > abs(W(i-1)))
      i=i-1; 
      if (i < 1) % Walking downward, take care!
	if (abs(W(points)) > abs(W(1)))
	  i = points - 1;
	else
	  i = 1;
	  break;
	end
      end
    end % while-loop
    nend = i;
  else
    % We're hopefully on the mainlobe!
    i = max(nend,2);
    while (abs(W(i)) < abs(W(i-1)))
      i=i+1;
      if (i > points) % Walking upward, take care!
	if (abs(W(1)) > abs(W(points)))
	  i = 2;
	else
	  i = points;
	  break;
	end
      end
    end %while-loop
    nend = i;
  end % if-else

  % Test the different cases to return correct values
  if ((nstart<n6dB_1)&(n6dB_1<n6dB_2)&(n6dB_2<nend))
    % All is OK
    return
  elseif ((n6dB_1<nstart)&(nstart<nend)&(nend<n6dB_2))
    % Find new values for n6dB_1 and n6dB_2
    n6dB = find(abs(W(nstart:nend)) > 10^(-6/20)); % indices of peak
    n6dB_1 = min(n6dB) + nstart - 1; 	% beginning of peak
    n6dB_2 = max(n6dB)+ nstart - 1;		% end of peak
    return
  else
     % When steereng is applied, the mainlobe can be "folded".
    fprintf('\nThe mainlobe is folded,'); 
    fprintf('\nBW and ISLR will not be correctly calculated');
    fprintf('\nNew code in findzero.m must be added\n');
  end
  
  clear W;


