%-----------------------------------------------------------------------------
% Function    : lag2D
%
% Description : Plot 2D image of beampattern of 2D array
%
%
% Language    : Matlab 5
% Written by  : Andreas Austeng/Sverre Holm, IFI
%
% Version no. : 1.0 AA 01.01.2000 first version
%                   SH 17.08.2000 Axis, labels added
%                   AA 18.08.2000 Cleaned up and added white circle
%                                 indicating physical angles.
%                                 Using caxis instead of max(...).
% 
% Called by   : resp2d
% Calling     : 
% -----------------------------------------------------------------------------
%
% Find beamwidth for 2D array (MxM elements)

function resp = lag2D(varargin)
%
% parameters:
  N = 512;       % N-pts fft
  dBlimit = -50; % minimum level in beampattern


error(nargchk(1,3,nargin))
  % Unpack and check input variables
  aa = varargin{1};

  if  nargin>1,
    phi = varargin{2}
  else
    phi=0;
  end
  if  nargin>2,
    theta = varargin{3}
  else
    theta=0;
  end
  if (theta ~= 0)| (phi~= 0),
    disp(['Steering angles (',num2str(phi),', ',num2str(theta),')'])
  end
  

  
  [M1,M2] = size(aa);
  
  sx = sin(phi)*cos(theta);
  sy = sin(phi)*sin(theta);

  
  steer = zeros(M1,M2);
  for k = 0:M2-1
    yy = pi*sy*k;
    for l = 0:M1-1
      xx = pi*sx*l;
      steer(l+1,k+1) = exp(-j*(xx + yy) );
    end
  end
  
  aa = aa.*steer;
  
  % Beregne respons
  bb = abs(fftshift(fft2(aa,N,N)));
  cc = 20*log10(bb/max(max(bb))+eps);
  
  colormap(jet);

  %  imagesc(max(dBlimit,cc));
  ax = linspace(-1,1,N);
  imagesc(ax,ax,cc);
  caxis([dBlimit 0]);
  axis square
  xlabel('u = sin(\phi) cos(\theta)')
  ylabel('v = sin(\phi) sin(\theta)')
  title('Array response')

  colorbar;
  
  % Circle indicating physical angles
  hold on
  plot(cos(pi*ax),sin(pi*ax),'w','LineWidth',1.5);
  hold off
  
  % Beregne -6dB contour:
%  hold on;
  [xx]=contourc(cc,[-6 -6],'b');   
%  hold off
  % Beregne strste BW
  resp = cc;
  
  yy=zeros(xx(2,1));
  for i=2:xx(2,1)+1
    yy(i-1,:)= sqrt((xx(1,i) - xx(1,2:xx(2,1)+1)).^2 + (xx(2,i) - xx(2,2:xx(2,1)+1)).^2);
  end
  
  BW = max(max(yy)); % BW i kontrollpunker
  
  bw = 180/pi*asin(BW/N*2); % bw i grader
  
  % Aproximerer 1. nullpkt med: 
  % Mainlobe is defined as 2*(-6dB beamwidth) 
  % (approx equal 3*(-3 dB beamwidth) as used by ESA for Satellite Synthetic
  % Aperture Radar image quality characterization)
  
  [x,y] = meshgrid(-(N/2-.5):1:(N/2-.5));
  r = sqrt(x.^2+y.^2)<=BW;
  
  ML = sum(sum(bb.*r));
  cc = bb.*(1-r);
  SL = sum(sum(cc));
  Peak = max(max(cc))/max(max(bb));
  Avg  = mean(mean(cc))/max(max(bb));
  ISLR = SL/ML;
  
  fprintf('\n  BW (-6dB) = %7.2f deg',bw);
  fprintf('\n  Peak      = %7.2f dB',20*log10(Peak));
  fprintf('\n  Avg       = %7.2f dB',20*log10(Avg));
  fprintf('\n  ISLR      = %7.2f dB\n\n',10*log10(ISLR));
  
  clear Avg Peak dBlimit steer y BW SL i sx yy ISLR aa k ...
      sy M1 ax l theta M2 bb phi varargin ML bw r x N cc xx 

  