%------------------------------------------------------------------------------
% Function    : resp2d(plotfig, weights, elem_pts, flagg, media, excitation,
%               transducer, u0_opt, u1_opt);
%
% Description : plots the far-field array response from a 2d array transducer.
%               weights are half the transducer weights by symmetry.
%               if isempty(weigths) then user defined apodization or
%               uniform weighting is used in computation - depending
%               on excitation(5) parameter. 
%
%
% Language    : Matlab 4.2
% Written by  : Bjrnar Elgetun, Ifi-UiO
%
% Version no. : 1.0 BJE 07.07.95 first version
%               1.1 BJE 11.07.95 added -6 dB bandwidth on uniform plot
%               1.2 BJE 22.08.95 changed # elements detection
%               1.3 BJE 21.09.95 added SL-estimate due to weighting
%               1.4 BJE 29.09.95 changed thinning calculation
%               1.5 BJE 11.10.95 changed plot-point region
%		1.6  SH 27.10.95 modified bw computation
%               1.7 BJE 01.11.95 correction when displaying for sin(phi)
%               1.8 BJE 02.11.95 check if weights exists, added transducer
%                                in call
%               1.9 BJE 03.11.95 set minimum plot level, and check if 
%                                user defined apodization is selected.
%               2.0 BJE 21.11.95 changed input
%               2.1 BJE 14.12.95 read oversampling rate from user
%		2.2  SH 20.02.96 Removed symmetry constraint
%               2.5  SH 11.04.96 flagg(4)=4 => (kx,ky) plot with steering
%                                + greek plot legends
%               2.6 SH  26.11.96 Output average sidelobe level
%                                Corrected peak to work on abs(W)
%               2.7 SH  20.12.96 Speeded up routine by doing calculations 
%                                according to need, output params for unweighted
%		2.8 NAA 18.02.97 k-space handles variable respons resolution
%               2.9 SH  22.04.97 Added missing variable T for title
%                   SH  17.08.00 Initalized weightoption, uniW, W
%                                removed 1 superfluous ends
%                                Choice of Image or mesh plot
%                                added 3 new variables in call
%
% Called by   : mslmain, wntmain, ffmain, annthin, optthin
% Calling     : setcomap, vwctrl, vwctrl2, clipl, vinput, yinput. 
%               lag2D, bp_wc
%               /local/matlab/toolbox/local/graphics/textfcns 
%------------------------------------------------------------------------------

function resp2d(plotfig, weights, elem_pts, flagg, media, excitation, ...
		transducer, u0_opt, u1_opt, ...
		config, configOrigHandle, plotfigOrigHandle);


%
% initialization of transducer
%
  fprintf('Calculating array response ..\n');

  active=find(elem_pts(4,:) > 0);	% find active transducer elements
  N_active=length(active); 		% # active elements
  N_x=transducer(2);			% # elements x direction, full
  N_y=transducer(6);			% # elements y direction, full
  N_full=length(elem_pts(1,:));

  xpos=elem_pts(1,active)'; 		% x-coord of elements
  ypos=elem_pts(2,active)'; 		% y-coord of elements

  uniweights=ones(N_active,1)./N_active; % uniform weight set

  mode = yinput('2D image plot instead of mesh (y/n)? ',1);

if mode == 1,
  figure(plotfig);
  cleanscr(plotfig,plotfigOrigHandle)
  A=zeros(N_x, N_y);
  A(active)=1;
  resp=lag2D(A);
  %
  % generate worst-case cut
  figure(config);
  cleanscr(config,configOrigHandle)
  bp_wc(resp); 
  grid on
  
else
  
%
% check for existence of greek text library:
%
  greek = (exist('xlabtex.m') == 2);

%
% lambda normalization
%
  xpos=xpos/media(1,2)*excitation(1);
  ypos=ypos/media(1,2)*excitation(1);
%
% Initalize:
%
  weightoption='w'; 		
  uniW = 0;
  W = 0;

%
  
% read min plot level
%
  min_level=vinput('minimum response level for plot in dB',-50,-6,-50);

  if ~isempty(weights)                          % double weight vector
     weights = [fliplr(weights') weights']';
  end


%
% determine plotoption
%
  if (isempty(weights) & excitation(5) ~= 4) 	% no user defined apodization
      plotoption='u';				% uniform only
      weights=uniweights;
  elseif ~yinput('Append response plot for uniform weights',0)
      plotoption='w';				% weighted only
  else
      plotoption='b';				% both
  end
%
% fill in missing weights
%

      if (isempty(weights) & excitation(5) == 4) % user defined apodization
      global amp_ud;

      if (isempty(amp_ud) | (size(amp_ud,2) ~= length(active)) )
          fprintf('\nError: amp_ud (user-definded weights) has incorrect format.\n');
          fprintf('       Perform new weighting or turn off user-defined apodization\n');     
          fprintf('       in beamforming submenu.\n\n');
	  return; 				% do nothing more !
      end
      
      weights=amp_ud(1,:)'; 			% pick out the weights
      weights=weights./sum(weights); 	        % normalization
      weightoption='u'; 			% user defined
    end    
	
if flagg(4) ~=4,
%
% initialization of plot region. The plot points are distributed
% uniformly in [u,theta] plane for u=sin(phi). Thus the plot points are
% spaced closer for small phi (due to beamwidth detection).
%
  points_u=vinput('Plotpoints in phi direction',1,inf,max(4*N_x,256));
  points_theta=vinput('Plotpoints in theta direction',1,inf,2*N_y+1);

  u0=0; 				% u-axs
  u1=max(1,u1_opt);			% [u0_opt,u1_opt] optimization interval
  h_u=(u1 - u0)/(points_u-1);
  theta0=-pi/2; 			% theta-axis
  theta1=pi/2;
  h_theta=(theta1 - theta0)/(points_theta-1);
  [P_u,P_theta]=meshgrid( u0:h_u:u1, ...
      theta0:h_theta:theta1 ); 		% plot point matrices u=sin(phi),theta coord
else

%
% k-space. k = (kx,ky) is defined with normalized coordinates,
% i.e. needs to be multiplied 2*pi/lambda to become the wavevector
%
% steering k-vector
%
	theta_s = excitation(3);
	phi_s   = excitation(4);
	kx_s = sin(phi_s)*cos(theta_s);
	ky_s = sin(phi_s)*sin(theta_s);
%
% initialization of plot region in k-space
%
  points_k=vinput('Plotpoints in kx/ky directions',1,inf,max(4*N_x,64));

	kx0 = -1;
	kx1 = 1;
	h_kx = (kx1-kx0)/points_k;
	ky0 = -1;
	ky1 = 1;
	h_ky = (ky1-ky0)/points_k;
	[P_kx, P_ky] = meshgrid(kx0:h_kx:kx1, ky0:h_ky:ky1);	

	disp([' Steering to (theta, phi) = (', num2str(180/pi*theta_s),' ,',num2str(180/pi*phi_s), ...
                       ') deg, (kx, ky) = (',num2str(kx_s),' ,',num2str(ky_s),')*2pi/lambda']) 
end
%
% Calcualations
%
  if (flagg(4) == 4),
%
% k-space
%

  if (plotoption == 'w' | plotoption == 'b'),
    W=zeros(size(P_kx)); 		% array response to optimized weights
    for n=1:N_active
      W=W+[ weights(n)*exp(j*2*pi*( ...
	      (P_kx-kx_s)*xpos(n) + (P_ky-ky_s)*ypos(n)) )];
    end	
  end
  
  if (plotoption == 'u' | plotoption == 'b'),
    uniW=zeros(size(P_kx)); 		% array response to uniform weights
    for n=1:N_active
      uniW=uniW+[ uniweights(n)*exp(j*2*pi*( ...
	      (P_kx-kx_s)*xpos(n) + (P_ky-ky_s)*ypos(n) ))];
    end	  
  end
  
else
  
%
% calculating the array pattern on the plot points,
% find actual sidelobe level on array pattern and
% calculate actual -6dB bw on the array pattern
%
  if (plotoption == 'w' | plotoption == 'b'),
    W=zeros(size(P_u)); 		% array response to optimized weights
    for n=1:N_active
      W=W+[ weights(n)*exp(j*2*pi*P_u.* ...
	      (xpos(n)*cos(P_theta)+ypos(n)*sin(P_theta)) )];
    end

    bw=zeros(points_u,1);
    nsl = 0;
    for m=1:points_theta 		% loop over all theta
      n=max(find( W(m,:) >= 10^(-6/20) )); % largest column index in [inf,-6dB]
      bw(m)=u0 + (n-1)*h_u; 		% index -> u
%
% Mainlobe is defined as 2*(-6dB beamwidth) 
% (approx equal 3*(-3 dB beamwidth) as used by ESA for Satellite Synthetic
% Aperture Radar image quality characterization)
% 
      nsl = max(n,nsl);               % find index for start of sidelobe region
    end
    nsl = 2.0*nsl;
    bw=2*180/pi*asin(bw); 		% u -> deg, (double-sided)
    
%   nsl=ceil((u0_opt-u0)/h_u) + 1; 	% column in W where opt. region starts
    plevel=20*log10(max(max(abs(W(:,nsl:points_u))))); % peak sidelobe level in dB
    average=20*log10(mean(mean(abs(W(:,nsl:points_u))))); % average sidelobe
  end
  
  if (plotoption == 'u' | plotoption == 'b'), 
    uniW=zeros(size(P_u));		% array response to uniform weights
    for n=1:N_active
      uniW=uniW+[ uniweights(n)*exp(j*2*pi*P_u.* ...
	      (xpos(n)*cos(P_theta)+ypos(n)*sin(P_theta)) )];
    end
    unibw=zeros(points_u,1);
    unsl = 0;
    for m=1:points_theta 		% loop over all theta
      unin=max(find( abs(uniW(m,:)) >= 10^(-6/20) )); % largest column index in [inf,-6dB]
      unibw(m)=u0 + (unin-1)*h_u; 	% index -> u
      unsl = max(unin,unsl); 		% find index for start of sidelobe region
    end
    unsl = 2.0*unsl;
    unibw=2*180/pi*asin(unibw);         % u -> deg, (double-sided)
    plevelu=20*log10(max(max(abs(uniW(:,unsl:points_u))))); % peak sidelobe level in dB
    averageu=20*log10(mean(mean(abs(uniW(:,unsl:points_u))))); % average sidelobe
  end
end


%
% clip matrices
%
  W=clipl(abs(W),10^(min_level/20));
  uniW=clipl(abs(uniW),10^(min_level/20));

%
% init plot window
%
  vwctrl('dismiss',plotfig);			% clear old view controls
  vwctrl2('dismiss',plotfig);
  figure(plotfig);
  setcomap(plotfig,5);				% cool

  if (plotoption == 'b') 			% plot both responses
      axes2p=subplot(2,1,2);
      set(axes2p,'position',[.15 .1 .7 .25]);
      axes1p=subplot(2,1,1);			% split window
      set(axes1p,'position',[.15 .6 .7 .25]);
    else
      axes1p=subplot(1,1,1);			% one window
      set(axes1p,'position',[.15 .2 .7 .5]);
  end

%
% plot array response to uniform weights
%
  if (plotoption == 'b' | plotoption == 'u')

       if (flagg(4) == 4)			% u,theta coordinates

	  surf( P_kx,P_ky,20*log10(abs(uniW)) );
	  axis([kx0 kx1 ky0 ky1 min_level 0]);
	  caxis([min_level 0]);
	  title('Unweighted beampattern')
	  if (greek),

%
% legend for resp2d plot for k-space
% using greek texts from 'textfcns'

	  	xlabtex([ntex('k'),stex('x'),gtex('l/2p')])
	  	ylabtex([ntex('k'),stex('y'),gtex('l/2p')])
	  else
		xlabel('kx*lambda/2pi');
		ylabel('ky*lambda/2pi');
	  end

       else					% phi,theta coordinates

	  mesh( asin(P_u),P_theta,20*log10(abs(uniW)) );
	  axis([asin(u0) asin(u1) theta0 theta1 min_level 0]);
	  caxis([min_level 0]);
	  xlabel('phi');
	  ylabel('theta');

          bandwidth = mean(unibw(find(unibw>0)));
       	  T=sprintf('Response to uniform weighted array   -6 dB BW: %4.2f [deg]',bandwidth);
      	  title(T)
       end
       zlabel('Response [dB]');
       grid;
%
% print statistics
%
      fprintf('\n\n---------------------------------------------------------\n');
      fprintf('Unweighted response calculation statistics:\n\n');
      fprintf('Actual Beamwidth (-6 dB) :       %10.2f [deg]\n',bandwidth);
      fprintf('Actual peak sidelobe level:      %10.2f [dB]\n',plevelu);
      fprintf('Actual average sidelobe level:   %10.2f [dB]\n',averageu);
      fprintf('---------------------------------------------------------\n');
   end

%%%% end

%
% plot array response to optimized weights and print statistics
%
  if (plotoption == 'b' | plotoption == 'w')

      if (plotoption == 'b')
	  axes(axes2p);
      end

      if (flagg(4) == 4) 			% kx,ky coordinates

	  surf( P_kx,P_ky,20*log10(abs(W)) );
	  axis([kx0 kx1 ky0 ky1 min_level 0]);
	  caxis([min_level 0]);
	  title('Weighted beampattern')

	  if (greek),
%
% legend for resp2d plot for k-space
% using greek texts from 'textfcns'

	  	xlabtex([ntex('k'),stex('x'),gtex('l/2p')])
	  	ylabtex([ntex('k'),stex('y'),gtex('l/2p')])
	  else
	  	xlabel('kx*lambda/2pi');
	  	ylabel('ky*lambda/2pi');

	  end

	else					% phi,theta coordinates

	  mesh( asin(P_u),P_theta,20*log10(abs(W)) );
	  axis([asin(u0) asin(u1) theta0 theta1 min_level 0]);
	  caxis([min_level 0]);
	  xlabel('phi');
	  ylabel('theta');
	  zlabel('Response [dB]');

          bandwidth = mean(bw(find(bw>0)));


%
% add text and grid
%
      if (weightoption == 'u')
	  T=sprintf('Response to user-defined weights    -6 dB BW: %4.2f [deg]',bandwidth);
	else
      T=sprintf(['Response optimized from: %4.2f [deg]   Peak:%4.1f [dB]   ',...
	    '-6 dB BW: %4.2f [deg]'], asin(u0_opt)*180/pi, plevel, bandwidth); 

      end

      title(T);


      end		

      grid;

%
% print statistics
%
      fprintf('\n\n---------------------------------------------------------\n');
      fprintf('Weighted response calculation statistics:\n\n');
      fprintf('Actual Beamwidth (-6 dB) :       %10.2f [deg]\n',bandwidth);
      fprintf('Actual peak sidelobe level:      %10.2f [dB]\n',plevel);
      fprintf('Actual average sidelobe level:   %10.2f [dB]\n',average);

      p=1-N_active/(N_full); 		% thinning
      if p>0
	fprintf('Theoretical average: 	         %10.2f [dB]\n',-10*log10(N_active));
      end
      fprintf('---------------------------------------------------------\n');
  end

%
% add viewcontrols
%
  if (plotoption == 'b')
      vwctrl2('init',plotfig,axes1p,axes2p);
    else
      vwctrl('init',plotfig);
  end

%
% and a final message
%
  if (weightoption == 'u')
      fprintf('\nFinished!\n\n')
  end

end % lag2D or resp2D