function cosFactor = cos_fact(x0,y0,z0,transducer,flagg,xobs,yobs,zobs,cosflagg)
%------------------------------------------------------------------------------
% Routine         : cosFactor = 
%                   cos_fact(x0,y0,z0,transducer,flagg,xobs,yobs,zobs,cosflagg)
% Description     : Calculate cos of the angle between the rays from the transducer
%                   (x0,y0,z0) and to the observation points (xobs,yobs,zobs),
%                   and the vector normal to the transducer at point (x0,y0,z0)
%                   cosflagg=1 => cos_corr=cos(angle)
%                   cosflagg=2 => cos_corr=(1+cos(angle))/2
% Language        : Matlab 5.0
% Written by      : Lars Odegaard
% Version no.     : 1.0 LO 8/6-98    First version
%                   1.1 LO 14/12-98  Elevation focus=Inf is ok 
%                                    Corrected sign on [nx,ny,nz] for annular array
% Called by       : pec
% Routines called : normal
%----------------------------------------------------------------------------- 

tmpflagg=0;

if flagg(2)==3  %annular transducer
   F=transducer(9);
   htype=[2,abs(F),abs(F),abs(F),sign(F),0];
   tmpflagg=1;
   [nx,ny,nz] = normal(x0,y0,z0,htype,0);
   nx=-nx;
   ny=-ny;
   nz=-nz;
end     


if flagg(2)==1 % 1D 1.5D og 2D rektangular transducer
   % Let f=f(x,y)=z be the surface of the transducer
   % The normal of the transducer is given by:
   % n=[-df/dx,-df/dy,1]=[dfdx,dfdy,1]=[nx,ny,nz]*nr
   
   % Calculate dfdx:
   % azimuth fixed focus
   Fxzplane=transducer(17); 
   if (Fxzplane==0)|(abs(Fxzplane==Inf)), % Flat transducer (example: FLA or FPA) 
      dfdx=0;   
   else
     htype_az=[2,abs(Fxzplane),Inf,abs(Fxzplane),sign(Fxzplane),0];
     [nx_az,ny_az,nz_az] = normal(x0,y0,z0,htype_az,0);
     dfdx=nx_az/nz_az;
   end
   
   % Calculate dfdy
   % elevation fixed focus
   Fyzplane=transducer(9);  
   if (transducer(6)==3)|(transducer(6)==5) % 3 or 5 elevation elements
      if (transducer(6)==3)&(abs(y0)>(transducer(12)/2))         
         Fyzplane=transducer(11);
      elseif (transducer(6)==5)&(abs(y0)>(transducer(12)/2+transducer(13)))
         Fyzplane=transducer(11)
      elseif (transducer(6)==5)&(abs(y0)>(transducer(12)/2))
         Fyzplane=transducer(10)
      else 
      end       
   end      
   if (Fyzplane==0)|(abs(Fyzplane==Inf)), % Flat 'linse' 
      dfdy=0;   
   else
     htype_el=[2,Inf,abs(Fyzplane),abs(Fyzplane),sign(Fyzplane),0];
     [nx_el,ny_el,nz_el] = normal(x0,y0,z0,htype_el,0);
     dfdy=ny_el/nz_el;
   end
   
   % Normalize vector normal to transducer at (x0,y0,z0))
   nr=sqrt(dfdx^2+dfdy^2+1^2);
   nx=dfdx/nr;
   ny=dfdy/nr;
   nz=1/nr;            
   tmpflagg=1;
end


% Calculate ray from transducer point (x0,y0,z0) and observation points 
% (xobs,yobs,zobs)
if tmpflagg==1
   xout = xobs - x0;  
   yout = yobs - y0;
   zout = zobs - z0;
   r = sqrt(xout.^2 + yout.^2 + zout.^2);
   xout = xout./r;
   yout = yout./r;
   zout = zout./r;
   if cosflagg==1
     cosFactor=xout.*nx + yout.*ny + zout.*nz;
   elseif cosflagg==2  
     cosFactor=(1+xout.*nx + yout.*ny + zout.*nz)/2;
   else
     cosFactor=1;  
     disp('Warning 1 from cos_fact: cosfactor not included')
   end     
else
   cosFactor=1;
   disp('Warning 2 from cos_fact: cosfactor not included')
end

   




