%----------------------------------------------------------------
% Routine     : imageplo.m (PE Color-encoded) 
%
% Description : Plots the results from a PE-analysis in the 
%               'plotfig' window as a Color-encoded plot.               
%               
% Language    : Matlab V4.1
% Written by  : Vingmed sound A/S, Trond Kleveland 
%
% Version no. : 1.0 HGJ 04.04.95 First version based on peplcont.m
%             : 1.1 BJE 23.04.95 Changed colormap -> setcomap()
%             : 1.2 SH  26.05.95 Removed top text window, added colorbar
%             : 1.3 SH  10.05.96 Added subplot command to initialize graph
%                                window
%               1.4 SH   9.10.96 Removed cleanscr command (often removed top
%                                command line in window)
%               1.5 SH  12.11.96 Improved check for y/t reflection of data
%               1.6 SH  22.05.97 Can now do 'iso' for pulse data
%               2.8 SH  29.05.97 Matlab 5: removed statement that gave warnings
%                                (test for isempty after input statement)
%               2.9 SH  15.03.99 iso-option: added abs() to peak finding
%                   SH  16.08.07 added extra figure(plotfig) command
%                                
% Called by   : usimcnf
% Calling     : cleanscr, petxt, setcomap
%----------------------------------------------------------------

function imageplo(resp, excitation, media, transducer, option, flagg, ...
       x, y, z, t, plotfig, plotfigOrigHandle)
  
%
% INITIALIZATION
%
  clc
  % why has cleanscr been removed? 16.08.2007
  cleanscr(plotfig,plotfigOrigHandle)
  figure(plotfig)
  subplot(1, 1, 1)

  MeshCont = 'Cont';
  Rf_Envelope = 'e';   
  
%
% set plot option : lin/log/iso
%

  disp('  ')
  disp('Set plot option : ')
  disp('  ')
  
  plotOption = input('-> Beamwidth / Linear / Logarithmic [iso/lin/log] ? ','s');
  if isempty(plotOption) | (all(plotOption ~= 'lin') & all(plotOption ~= 'log'))
      plotOption = 'iso';       % default
  end
  disp('  ')
  
%
% set the text variables
%
  [Xtxt,Ytxt,Ztxt,Ttxt1,Ttxt2,Ttxt3] = ...
      petxt(excitation,media,transducer,option,flagg,z,t,Rf_Envelope,plotOption,MeshCont);
%
% axis, depending on distance or time analysis
%
  mirror = 0;
  if all(option == '2xz')         
      Xx = x(1,:)*1e3;
      Zz = z(:,1)'*1e3;
      mirror = all(max(Xx')==0) | all(min(Xx')==0); 
    elseif all(option == '2xy') 
      Xx = x(1,:)*1e3;
      Zz = y(:,1)'*1e3;
    elseif all(option == '2yz')
      Xx = y(1,:)*1e3;
      Zz = z(:,1)'*1e3;
      mirror = all(max(Xx')==0) | all(min(Xx')==0); 
    elseif all(option == '2xt')
      Xx = x(1,:)*1e3;
      Zz = t(:,1)'*1e6;
      mirror = all(max(Xx')==0) | all(min(Xx')==0); 
  end
  
  if mirror,
%
% this question is only asked if the plot is (x,z), (y,z), or (x,t) and
% 0 is contained in the first axis.
%
      reflect = input('-> Reflect plot about z/t-axis [y/n] ? ','s');   
      if isempty(reflect)
          reflect = 'y';
      end
      if reflect == 'y',
          LXx = length(Xx);
          respud = flipud(resp);        
          if (max(Xx) > 0),
              Xx = [-fliplr(Xx(2:LXx)), Xx];
              resp = [respud(1:LXx-1,:)', resp']';
            else
              Xx = [Xx, -fliplr(Xx(1:LXx-1)) ];
              
              resp = [resp', respud(2:LXx,:)']';
          end
          
      end
  end
  
%
% Find the envelope of the signal if we have specified that kind of 
% computation
%
  if all(plotOption == 'lin') 
      
      resp=abs(resp);
      
    elseif all(plotOption == 'log') 
      
      resp=20*log10(abs(resp)/transducer(2) + eps);
      
    elseif all(plotOption == 'iso')
      
      [rows, cols] = size(resp);
      
      for depth = 1:cols
	beam_max = max(abs(resp(:,depth)));
   if (beam_max > 0)
      resp(:,depth) = resp(:,depth)/beam_max;
	end
      end
      resp=20*log10(abs(resp) + eps);
      
    else
      
      disp('ERROR detected in imageplo')
      disp('  ')
      return
      
  end
%
% get current axes position modify it and make up another axes for text
%
  
% added 16.08.2007: had lost track of which figure and plotted in 'config' instead
    figure(plotfig)
    
    handle1 = gca;
    v1 = get(handle1,'Position');
    v1(4)=v1(4)-0.1;
    set(handle1,'Position',v1)
  
    v2    = v1;
    v2(1) = v2(1) - 0.1;
    v2(2) = v2(2) + v2(4) + 0.05;
    v2(4) = 0.1;
    cla
    handle2 = axes('Position',v2);
    axis('off')
    cla

%
% Plot the computed response in a Color-encoded
%
    axes(handle1)
    cla

  imagesc(Zz,Xx,resp);
  
%
% sets colormap and update menu checking
%
%  setcomap(plotfig,8);		% jet 
  
  xlabel(Xtxt);
  ylabel(Ytxt);
  
  colorbar


    axes(handle2)
    cla
    text(0 , 1.0 ,Ttxt1);
    text(0 , 0.5 ,Ttxt2);
    text(0 , 0.0 ,Ttxt3);
  
  fprintf('\n\nFinished!\n');
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
