% -----------------------------------------------------------------------------
% Function    : [elem_pts,centers]=thin(transducer,flagg,elem_pts,centers, ...
%                config,usr_resdir, separator)
%
% Description : Random thinning of array elements. The elements are picked from
%               either a uniform or a gaussian distribution. Elements may be
%               picked symmetrically as follows
%               - No symmetry
%               - Bi-symmetry e(m,n)=e(-m,-n)
%               - Quad-symmetry e(m,n)=e(-m,n)=e(-m,-n)=e(m,-n)
%               Binned thinning means that if 25% shall be kept from a 128
%               element array, 1 element is placed in [1..4], 1 in [5..8]
%               etc according to the probability distribution
%
%
% Language    : Matlab 4.2
% Written by  : Bjrnar Elgetun, Ifi-UiO
%
% Version no. : 1.0 BJE 27.09.95 first version
%               1.1 SH  03.06.96 thinning of elements, not just points
%               1.2 AA  19.03.97 keeping the two outer elements
%                                for unsymmetric thinning
%               1.3 SH  11.07.97 Binned vs random thinning for 1D asymmetric
%                                array, triangular distribution
%                                added storage of parameters,
%                                usr_resdir, separator in call
%               1.4 SH  21.07.97 Rearranged, put thinning in function randthin
%                   SH  28.07.97 Minor changes in user interface etc 
%                   SH  10.09.97 Changed call to randthin, binned 2D thinning
%               7.0 SH  01.03.05 ''' in strings to accept filenames with
%               spaces
%
% Called by   : usimcnf
% Calling     : viewtrsd, randthin
% -----------------------------------------------------------------------------

function [elem_pts,centers]=thin(transducer,flagg,elem_pts,centers, ...
          config,usr_resdir,separator);


 

setup = [usr_resdir separator 'thinpar.mat'];
path(path,usr_resdir);

  if (exist('thinpar.mat')==2),
    eval(['load ''',setup,''';']);
  else
%
% INITIALIZE
%  
    thinpr(1) = 1;		% default gaussian distribution
    thinpr(2) = 2;		% default Bi-symmetry
    thinpr(3) = 1;               % default random thinning
  end  
  dist     = thinpr(1);		
  symmetry = thinpr(2);			
  meth     = thinpr(3);                 
  
  N               = length(centers(1,:));
  centers(4:5,:)  = abs(centers(4:5,:));      % restore all elements
  elem_pts(4:5,:) = abs(elem_pts(4:5,:));     % and points
  viewtrsd(transducer,elem_pts,flagg);  
  
%
% detect transducer
%
  if ~all(centers(3,:) == 0)           % z-coordinate elements
      trsd=3;
      if (transducer(18) == 0)
	  trsdStr='3D rectangular array';
	else
	  trsdStr='3D elliptic array';
      end

    elseif ~all(centers(2,:) == 0)	% y-coordinate elements
      trsd=2;
      if (transducer(18) == 0)
	  trsdStr='2D rectangular array';
	else
	  trsdStr='2D elliptic array';
      end

    else          			% z-coordinate elements
      trsd=1;
      trsdStr='1D linear array';
  end


%
% READ USER INPUT
%
  finished=0;
  while ~finished

    if (dist == 2)
      distributionStr='Uniform';
    elseif (dist == 3)
      distributionStr='Triangular';
    else
      distributionStr='Gaussian';
    end
    
    if (symmetry == 1)
      symmetryStr='No';
    elseif (symmetry == 2)
      symmetryStr='Bi [e(m,n)=e(-m,-n)]';
    elseif (symmetry == 4)
      symmetryStr='Quad [e(m,n)=e(-m,n)=e(-m,-n)=e(m,-n)]';
    end
    
    if (meth == 2)
      methodStr = 'Binned';
    else
      methodStr = 'Random';
    end
      
    clc;
    fprintf('\nTHINNING TRANSDUCER ELEMENTS\n\n\n')
    fprintf('Transducer   : 		%s\n',trsdStr);
    fprintf('Elements     : 		%i\n\n',N);
    
    fprintf('Method       : 		%s\n',methodStr);
    fprintf('Distribution : 		%s\n',distributionStr);
    fprintf('Symmetry     : 		%s\n\n\n',symmetryStr);
    
    if ~strcmp(input('-> Accept parameters [y/n] ? ','s'),'n') % default 'y'
      finished=1;
    else
      
      temp=input('-> New method (random/binned) [r/b] ? ','s');
      if strcmp(temp,'b')
	if (trsd ~=1 & symmetry ~= 1)
	  disp('Error, binned method only for asymmetric 1D arrays!')
	  pause(2)
	  meth = 1;
	else  
	  meth = 2;
	end
      elseif strcmp(temp,'r')
	meth = 1;
      end
      
      temp=input('-> New distribution (Gaussian/uniform/triangular) [g/u/t] ? ','s');
      if strcmp(temp,'u')
	dist = 2 ;
      elseif strcmp(temp,'t')
	dist = 3;
      elseif strcmp(temp,'g')
	dist = 1;
      end
	  
      temp=input('-> New symmetry (none/bi/quad) [n/b/q] ? ','s');
      if strcmp(temp,'n')
	symmetry=1;
      elseif (strcmp(temp,'q') & trsd > 1) % require 2D or higer dimension
	symmetry=4;
      elseif strcmp(temp,'b')
	symmetry=2; 	
      end
      
    end 				% if accept
  end 					% while ~finished

%
% save setup
%

thinpr = [dist symmetry meth];
vers = version;
if vers(1)=='5'
    
  eval(['save ''',setup,''' thinpr -v4']);
else
  eval(['save ''',setup,''' thinpr;']);
end

if (dist == 3)
   distribution = 't';
elseif (dist == 2)
   distribution = 'u';
else
   distribution = 'g';
end

if (meth == 2)
   method = 'b';
else
   method = 'r';
end

  %
  % read distribution width if gaussian
  %
  if (distribution == 'g')
      w_SD=input('-> Enter distribution width in SD (3) : ');
      if isempty(w_SD)
	  w_SD=3;		% default
      end
  end
  
  %
  % read and set seed
  %
  seed=input('-> Enter seed (sum(100*clock)) : ');
  if (isempty(seed))
      seed=sum(100*clock);	% default
  end
  randn('seed',seed);	% gaussian
  rand('seed',seed);	% uniform and triangular

  %
  % read legal number of elements to remove
  %
  finished=0;
  if ~(trsd == 2 & method =='b') 

    while ~finished		% check input
      N_to_thin=N-input('-> Enter number of elements to keep: ');

      if (isempty(N_to_thin))
	  fprintf('Error: Missing number\n');
	elseif (N-N_to_thin < 0)
              disp('Returning to main ultrasim session')
              return
            elseif (N_to_thin > N-2*symmetry)
	  fprintf('Error: Number outside interval [ %i,%i]\n',0,N-2*symmetry);
	elseif ( symmetry == 2 & ceil(N_to_thin/2) ~= floor(N_to_thin/2) )
	  fprintf('Error: Number must have a factor of 2\n');
	elseif ( symmetry == 4 & ceil(N_to_thin/4) ~= floor(N_to_thin/4) )
	  fprintf('Error: Number must have a factor of 4\n');
	elseif ( method == 'b' & rem(N,N-N_to_thin))
	  fprintf('Error: Number of elements must be an integer factor of %i\n', N)
	else
	  finished=1;
      end
    end
    N_to_thin2 = 1;

  else % binned 2D
    N1 = max(abs(centers(4,:)));
    N2 = max(centers(5,:));
    if (N1*N2 > length(centers)),
      disp('For a non-rectangular array,')
      disp('number of elements refers to an imaginary rectangular array')
    end

    finished = 0;
   while ~finished		% check input
    N_to_thin=N1-input(['-> Enter number of elements to keep in x-direction (', ...
              num2str(N1),') : ']);
    N_to_thin2=N2-input(['-> Enter number of elements to keep in y-direction (', ...
              num2str(N2),') : ']);
        if (isempty(N_to_thin) | isempty (N_to_thin2))
	  fprintf('Error: Missing number\n');

	elseif (N1-N_to_thin < 0 | N2-N_to_thin2 < 0)
              disp('Returning to main ultrasim session')
              return
	elseif ( method == 'b' & rem(N1,N1-N_to_thin))
	  fprintf('Error: Number of elements in x-direction must be an integer factor of %i\n', N1)
	elseif ( method == 'b' & rem(N2,N2-N_to_thin2))
	  fprintf('Error: Number of elements in y-direction must be an integer factor of %i\n', N2)
        else
          finished = 1;
        end

      N=N1*N2;
      end
  end


%
% THINNING
%

  if (N_to_thin*N_to_thin2/N > .9)
      fprintf('\n\nThinning %4.1f %s .. (This can take a looong time)\n\n', ...
          N_to_thin*N_to_thin2/N*100,'%');
    else
      fprintf('\n\nThinning %4.1f %s .. \n\n',N_to_thin*N_to_thin2/N*100,'%');
  end
  
  centers=randthin(centers, N_to_thin, N_to_thin2, method, distribution, ...
                          symmetry, w_SD, trsd);
  
%
% translate from centers to elem_pts
%
  [elem_pts]=centelem(centers,elem_pts);

  viewtrsd(transducer,elem_pts,flagg);  
  fprintf('%i elements left after thinning.\n', ...
      length(find(centers(4,:) >= 0)));
  fprintf('\n\nFinished thinning.\n');

