%----------------------------------------------------------------
% Routine     : petxt.m 
% Description : Makes up the text strings needed for the PE-plots
% Language    : Matlab V4.1
% Written by  : Vingmed sound A/S, Trond Kleveland 
%
% Version no. : 1.0 TKL 21.09.93, First version 
%             : 1.1 TKL 22.09.93, Removed analysis
%             : 1.2 TKL 23.09.93, Added time-analysis
%             : 1.3 SH  19.11.93, Added text for LinLog = 'Iso'
%             : 1.4 TKL 19.11.93, Replaced LinLog -> plotOption
%             : 1.5 TKL 24.11.93, Added yz-plane to analysis
%             : 1.6  SH  3.12.93, Annular array
%             : 1.7 TKL 13.12.93, Added xy-plane to analysis
%             : 1.8 SH  23.08.94, Changed text for y-z distance plot 
%             : 1.9 BJE 17.10.94, Changed plotOption
%	      : 2.0 BJE 21.11.94, Compatibility with movieflag - option(1)
%             : 2.1 SH  12.04.95, excitation(2:4) now in polar coordinates
%             : 2.2 KE   2.05.96, Added X & Y axis settings for 3D observ.
%               2.3 GHJ/SH 17.10.96 Text for Bessel beams
%               2.4 SH  11.11.96  Updated Bessel texts
%               2.5 SH  25.03.97  Conical transducer 
%               2.6 SH  02.07.97  Shows correct pitch for CLAs
%               2.7 SH  01.09.97  Corrected text for beamwidth plots
%               2.8 SH  07.04.98  Added text for tissue attenuation
%               2.9 SH  28.07.98  Double use of variable alfa corrected
%
%
% Called by   : peplot, peplcont
%----------------------------------------------------------------

function [Xtxt,Ytxt,Ztxt,Ttxt1,Ttxt2,Ttxt3] = ...
         petxt(excitation,media,transducer,option,flagg,z,t,Rf_Envelope,plotOption,MeshCont);

%
% EXCITATION
%
  osc = excitation(9);        % osc - transmitted pulse length (#osc)
  Weig_numb = excitation(10); % Weig_numb - pulse weighing (Rect=0,Cos=1)
  f = excitation(1);          % transmit frequency
  r = excitation(2);
  theta = excitation(3);
  phi = excitation(4);
  apod_numb_az = excitation(5);
  apod_numb_el = excitation(6);
%
% RECTANGULAR TRANSDUCER
%
  d  = transducer(1);
  de = transducer(5);
  N  = transducer(2);
  M  = transducer(6);
  order= transducer(12); % Bessel transducer
  alpha = transducer(13); % Bessel transducer
  deltax=d/N;
  ROC = abs(transducer(17));
  if (ROC ~= 0)
    arc = ROC*2*asin(d/(2*ROC));
    deltax = arc/N;
  end
%
% ANNULAR ARRAY
%
  FF = transducer(9);
%
% MEDIUM
%
  c = media(1,2);
  lambda =c/f;
  alfa  = media(1,9)/100;  % 1/cm
  beta  = media(1,10);
  freq  = f/1e6;
  att = exp(-alfa*freq^beta); 		% per cm
  dB = -10*log10(att)/freq;   % dB/cm/MHz
%  
% COMPUTED PARAMETERS
%
  plen = excitation(9)/excitation(1);  % plen = osc/f - pulse length [s]
%
% reference for distance- or time-based analysis
%
  if option(3) == 'y' | option(3) == 'z'          
      Distance_Time  = 'Dist';
      t_ref=t(1);
      r_ref = c * t_ref;
    else
      Distance_Time  = 'Time';   
      r_ref = z(1);
      t_ref = r_ref / c;
  end
%
% TEXT SECTION
%
  
%
% text for pulse-type/shape
%
  if (Rf_Envelope == 'e') & (Weig_numb == 1)
      pulse_txt='  Weighted-envelope';
    elseif (Rf_Envelope == 'e') & (Weig_numb == 0)
      pulse_txt='  Envelope';
    elseif (Rf_Envelope == 'r') & (Weig_numb == 1)
      pulse_txt='  Weighted-pulse';
    elseif (Rf_Envelope == 'r') & (Weig_numb == 0)
      pulse_txt='  Pulse';
  end
%
% text for apodization
%
  [apda_txt,apde_txt] = apodtxt(apod_numb_az,apod_numb_el);
%
% text for X-axis will specify distance or time analysis and focus
%
  
  if all(Distance_Time == 'Dist')
      if option(3) == 'z'
          if all(MeshCont == 'Mesh')
              Xtxt = ['Range in [mm], Azimuth focus=',num2str(r*1000),' [mm]  '];
            elseif all(MeshCont == 'Cont')
              Xtxt = ['Range in [mm], Azimuth focus=',num2str(r*1000), ...
                     ' [mm]  ',pulse_txt];
              if all(option(1:2) == '2y') | all(option(1:2) == 'my') 
                  if (transducer(6) == 1)	% 1 element in elevation
                      Xtxt = ['Range in [mm], Elevation focus=', ...
                             num2str(1000*transducer(9)), ...
                             ', Azimuth focus=',num2str(r*1000),' [mm]  '];
                    elseif (transducer(6) == 3)	% 3 elevation elements
                      Xtxt = ['Range in [mm], Inner, outer focus, offset=', ...
                             num2str(1000*transducer(9)), ...
                             ', ',num2str(1000*transducer(11)) , ...
                             ', ',num2str(transducer(16)*1000),'  [mm]  '];
                  end
              end   
          end
        elseif option(3) == 'y'
          if all(MeshCont == 'Mesh')
              Xtxt = ['Aperture (EL)   r=',num2str(r*1000),' [mm]  '];
            elseif all(MeshCont == 'Cont')
              Xtxt = ['Aperture (EL)   r=',num2str(r*1000), ...
                     ' [mm]  ',pulse_txt];
          end
      end
      fig_txt = ['Reference=',num2str(t_ref*1e6),' [us]'];       
    elseif all(Distance_Time == 'Time')
      if all(MeshCont == 'Mesh')
          Xtxt = ['Time in [us]    r=',num2str(r*1000),' [mm]  '];
        elseif all(MeshCont == 'Cont')
          Xtxt = ['Time in [us]    r=',num2str(r*1000), ...
                 ' [mm]  ',pulse_txt];
      end
      fig_txt = ['Reference=',num2str(r_ref*1e3),' [mm]'];       
  end
  
%
% text for Y- and Z- axis
%
  
%
% linear plot
%

  
  if all(plotOption == 'lin')
      if all(MeshCont == 'Mesh')
% ----------------------------
% Added for 3D plot by KE
	  if (option(1) == '3') | (option(1) == 'f')
	     Ytxt = ['Aperture (AZ)  d=',num2str(d*1e3),' [mm]'];
% ----------------------------
          elseif option(2) == 'x'          
              Ytxt = ['Aperture (AZ)  d=',num2str(d*1e3),' [mm]'];
          elseif option(2) == 'y'          
              Ytxt = ['Aperture (EL)  d=',num2str(de*1e3),' [mm]'];
          end
% ----------------------------
% Added for 3D plot by KE
	  if (option(1) == '3') | (option(1) == 'f')
	     Ztxt = ['Range in [mm], Elevation focus=',num2str(theta*1000),' [mm]'];
% ----------------------------
          else
             Ztxt = ['RESPONSE  [lin]   ',pulse_txt];
          end
        elseif all(MeshCont == 'Cont')
          if option(2) == 'x'      
    
              Ytxt = ['CONTOUR PLOT [lin]  Aperture (AZ) d=',num2str(d*1e3),' [mm]'];
            elseif option(2) == 'y'          
              Ytxt = ['CONTOUR PLOT [lin]  Aperture (EL) d=',num2str(de*1e3),' [mm]'];
          end
          Ztxt = [' '];
      end
%
% log plot
%
    elseif all(plotOption == 'log')
      if all(MeshCont == 'Mesh')
          if option(2) == 'x'          
              Ytxt = ['Aperture (AZ)  d=',num2str(d*1e3),' [mm]'];
            elseif option(2) == 'y'          
              Ytxt = ['Aperture (EL)  d=',num2str(de*1e3),' [mm]'];
          end
          Ztxt = ['RESPONSE [0 -50dB]',pulse_txt];
        elseif all(MeshCont == 'Cont')
          if option(2) == 'x'          
              Ytxt = ['CONTOUR PLOT [dB]  Aperture (AZ) d=',num2str(d*1e3),' [mm]'];
            elseif option(2) == 'y'          
              Ytxt = ['CONTOUR PLOT [dB]  Aperture (EL) d=',num2str(de*1e3),' [mm]'];
          end
          Ztxt = [' '];
      end
%
% iso plot
%
    elseif all(plotOption == 'iso')
      if all(MeshCont == 'Mesh')
          if option(2) == 'x'          
              Ytxt = ['Aperture (AZ)  d=',num2str(d*1e3),' [mm]'];
            elseif option(2) == 'y'          
              Ytxt = ['Aperture (EL)  d=',num2str(de*1e3),' [mm]'];
          end
          Ztxt = ['ISO RESPONSE [0 -50dB]',pulse_txt];
        elseif all(MeshCont == 'Cont')
          if option(2) == 'x'          
              Ytxt = ['BEAMWIDTH [mm], Aperture (AZ) ',num2str(d*1e3),' mm'];
            elseif option(2) == 'y'          
              Ytxt = ['BEAMWIDTH [mm], Aperture (EL) ',num2str(de*1e3),' mm'];
              if all(option(1:2) == '2y') | all(option(1:2) == 'my') 
                  if (transducer(6) == 3)	% 3 elevation elements
                      Ytxt = ['BEAMWIDTH [mm], Aperture (EL) ', ...
                             num2str(transducer(12)*1e3),'+2*', ...
                             num2str(transducer(14)*1000),' mm'];
                  end
              end       
          end
          Ztxt = [' '];
      end
  end
  
%
% text for the title field
%
  Ttxt1 = ['ARRAY-RESPONSE','   ',fig_txt,'   ',datetime];
  
  if flagg(2) == 1
%
% rectangular array
%
      Ttxt2 = ['Theta=',num2str(theta/2/pi*360), ...
             ' [deg]   Phi=',num2str(phi/2/pi*360),' [deg]  N=', ...
             num2str(N),'  M=',num2str(M), ...
             '   f=',num2str(f*1e-6),' [MHz]  pitch=', ...
             num2str(deltax/lambda),'   osc=',num2str(osc), ...
	     '   atten=',num2str(dB),' [dB/cm/MHz]'];
      Ttxt3 = [sprintf(apda_txt),'      ',sprintf(apde_txt)];
      
    elseif flagg(2) == 3
%
% annular array
%
      Ttxt3 = ['FF = ',num2str(FF*1000), ...
             ' [mm], f = ',num2str(f*1e-6),' [MHz],  osc = ',num2str(osc), ...
	     '  atten=',num2str(dB),' [dB/cm/MHz]'];
      
      if transducer(18) == 0

          Ttxt2 = ['Equal area annular array, N = ', num2str(N),' rings'];
        elseif (transducer(18) == 1)
          Ttxt2 = ['Equal width annular array, N = ', num2str(N),' rings'];
        elseif (transducer(18) == 2)
          Ttxt2 = ['Annular array with N = ', num2str(N),' elements on circumference'];

        elseif (transducer(18) == 4)
          Ttxt2 = ['Exact Bessel(',num2str(order), ...
		  ') beam, alpha = ',num2str(alpha),' m^-1'];        
        elseif (transducer(18) == 5)
          Ttxt2 = ['Equal-area Bessel(',num2str(order),') transducer, alpha = ', ...
                  num2str(alpha),' m^-1, N = ',num2str(N),' rings']; 
        elseif (transducer(18) == 6)
          Ttxt2 = ['Bessel(',num2str(order),') transducer, alpha = ', ...
                  num2str(alpha),' m^-1, N = ',num2str(N),' rings']; 
        elseif (transducer(18) == 7)
          Ttxt2 = ['4th-derivative bowtie Bessel beam, alpha = ',num2str(alpha),' m^-1'];       
        elseif (transducer(18) == 8)
          Ttxt2 = ['10th-derivative bowtie Bessel beam, alpha = ',num2str(alpha),' m^-1'];       
        elseif (transducer(18) == 9)
          Ttxt2 = ['Exact non-rotating Bessel(',num2str(order), ...
		  ') beam, alpha = ',num2str(alpha),' m^-1']; 
	  
        elseif (transducer(18) == 10)
          Ttxt2 = ['Conical, angle ',num2str(transducer(12)*180/pi),' deg'];       
        else
          Ttxt2 = ['Annular array of unknown type'];
      end
  end
  
  
  
  
  
  
